import os

if __name__ == '__main__':
    """
    genome annotation tools:

    MakerPlantRepeat
    MergeMaker
    MakerParser
    MapOldAnno
    GeneStruComp
    WiseAnno
    FamilyAnno
    FunctAnno
    AnnoEval
    """

    import argparse
    import configparser

    # command argument parse

    # argument parse
    parser = argparse.ArgumentParser(
        prog='GenomeAnnotation', description='tools help to get genome annotation more easy\n'
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for MakerPlantRepeat
    parser_a = subparsers.add_parser('MakerPlantRepeat',
                                     help='easy tools for running maker_p full repeat annotation pipeline',
                                     description='rewrite from http://weatherby.genetics.utah.edu/MAKER/wiki/index.php/Repeat_Library_Construction-Advanced')

    parser_a.add_argument("-c", "--config_file",
                          help="Path of configure file", type=str)
    parser_a.add_argument("-g", "--genome", help="genome fasta file", type=str)
    parser_a.add_argument(
        "-t", "--tag", help="tag for name, default as genome", default="genome", type=str)
    parser_a.add_argument("-n", "--num_threads",
                          help="num of threads", type=int)
    parser_a.add_argument("-s", "--split_genome",
                          help="split genome to contig to run, defaults will not", action='store_true')
    parser_a.add_argument("-o", "--output_dir",
                          help="path of output_dir, defaults as pwd", type=str)

    # argparse for EasyMaker
    parser_a = subparsers.add_parser('EasyMaker',
                                     help='easy tools for running maker',
                                     description='rewrite from https://weatherby.genetics.utah.edu/MAKER/wiki/index.php/MAKER_Tutorial_for_WGS_Assembly_and_Annotation_Winter_School_2018')

    parser_a.add_argument("-j", "--job", help="what job you want do? (default:first_anno)",
                          default="first_anno", choices=['first_anno', 'to_next_round', 'improve_anno', 'evaluate_anno'])
    parser_a.add_argument("-c", "--config_file",
                          help="Path of configure file", type=str)
    parser_a.add_argument("-g", "--genome_file",
                          help="genome fasta file", type=str)
    parser_a.add_argument(
        "-t", "--tag", help="tag for name, default as genome", type=str)
    parser_a.add_argument("-n", "--num_threads",
                          help="num of threads", type=int)
    parser_a.add_argument("-s", "--split_genome",
                          help="split genome to contig to run, defaults will not", action='store_true')
    parser_a.add_argument("-d", "--work_dir",
                          help="path of output_dir, defaults as maker_dir", default="maker_dir", type=str)

    # argparse for Protein2Genome
    parser_a = subparsers.add_parser('Protein2Genome',
                                     help='get gene structure from homo protein',
                                     description='get gene structure from homo protein')

    parser_a.add_argument('genome_file', type=str,
                          help='Path of target geonome fasta file')
    parser_a.add_argument('protein_fasta', type=str,
                          help='Path of query protein fasta file')
    parser_a.add_argument("-o", "--output_prefix", help="Output prefix name (default as Protein2Genome)",
                          default="Protein2Genome.gff3", type=str)
    parser_a.add_argument("-l", "--log_file",
                          help="path for log file (default as None)", type=str)
    parser_a.add_argument("-t", "--tmp_path", help="path for tmp dir (default in dir which have genome file)",
                          default=None, type=str)
    parser_a.add_argument(
        "-n", "--num_threads", help="cpu number (default as 10)", default=10, type=int)
    parser_a.add_argument("-c", "--gene_coverage", help="lowest gene coverage for genblasta hit use (default as 0.5)",
                          default=0.5, type=float)
    parser_a.add_argument("-a", "--genblasta_hit_num", help="geneblasta hit number (default as 20)", default=20,
                          type=int)
    parser_a.add_argument("-f", "--flank_range", help="flank range for genewise (default as 1000)", default=1000,
                          type=int)

    # argparse for GeneStruComp
    parser_a = subparsers.add_parser('GeneStruComp',
                                     help='Compare two different genetic structure annotations',
                                     description='Compare two different genetic structure annotations')

    parser_a.add_argument("gff_file1", help="gff file 1", type=str)
    parser_a.add_argument("gff_file2", help="gff file 2", type=str)
    parser_a.add_argument("-1", "--gene_list_1", help="List of genes intended for comparison 1, separated by commas",
                          type=str)
    parser_a.add_argument("-2", "--gene_list_2", help="List of genes intended for comparison 2, separated by commas",
                          type=str)

    # argparse for AnnoCleaner
    parser_a = subparsers.add_parser('AnnoCleaner',
                                     help='clean low annotated gene model\n',
                                     description='')

    parser_a.add_argument('gene_gff', type=str,
                          help='Path of genome feature gff file')
    parser_a.add_argument('output_file', type=str, help='Path of output file')
    parser_a.add_argument('-n', '--nr_bls', type=str,
                          help='Path of genes nr blast results outfmt6')
    parser_a.add_argument('-i', '--interpro_tsv', type=str,
                          help='Path of interpro results tsv file')
    parser_a.add_argument('-r', '--repeat_gff_file', type=str,
                          help='Path of repeat feature gff3 file')
    parser_a.add_argument('-e', '--entropy_tsv', type=str,
                          help='Path of entropy_tsv out file')
    parser_a.add_argument('-t', '--tran_evidence_gff', type=str,
                          help='Path of transcriptome map results gff file from pasa and cufflinks')
    parser_a.add_argument('-tt', '--tran_threshold', type=float,
                          help='high than this threshold will be think have rna evidence', default=0.5)
    parser_a.add_argument('-rt', '--repeat_threshold', type=float,
                          help='high than this threshold will be think a repeat range', default=0.5)
    parser_a.add_argument('-et', '--entropy_threshold', type=float,
                          help='small than this threshold will be think a low information sequence', default=3.9)

    # argparse for MapOldAnno
    parser_a = subparsers.add_parser('MapOldAnno',
                                     help='Move annotation results from one assembled version to another')

    parser_a.add_argument("old_genome", help="old genome fasta file", type=str)
    parser_a.add_argument("old_gff", help="old genome gene gff file", type=str)
    parser_a.add_argument("new_genome", help="new genome fasta file", type=str)
    parser_a.add_argument(
        "cDNA_gff", help="blat2gff.pl < Gel.yuan.cDNA.psl > Gel.yuan.cDNA.gff", type=str)
    parser_a.add_argument("output_gff", help="output gff file", type=str)
    parser_a.add_argument(
        "-t", "--tmp_dir", help="path of temp dir", type=str, default=None)
    parser_a.add_argument("-n", "--num_threads",
                          help="number of threads n=10", type=int, default=10)

    # argparse for MergeMaker
    parser_a = subparsers.add_parser('MergeMaker',
                                     help='merge maker output (for genome which split to 1M)')

    parser_a.add_argument(
        "maker_gff1", help="maker output gff (no seq) 1", type=str)
    parser_a.add_argument(
        "maker_gff2", help="maker output gff (no seq) 2", type=str)
    parser_a.add_argument("results_gff", help="merged results gff", type=str)
    parser_a.add_argument("evidence_gff", help="merged evidence gff", type=str)
    parser_a.add_argument(
        "ref_genome", help="reference genome fasta file", type=str)
    parser_a.add_argument(
        "-t", "--tmp_dir", help="path of temp dir", type=str, default=None)
    parser_a.add_argument("-n", "--num_threads",
                          help="cpu numbers", type=int, default=None)

    # argparse for MaskFromMaker
    parser_a = subparsers.add_parser('MaskFromMaker',
                                     help='get masked genome from maker gff file')

    parser_a.add_argument(
        "maker_gff", help="maker output gff (no seq) 1", type=str)
    parser_a.add_argument(
        "ref_genome", help="reference genome fasta file", type=str)
    parser_a.add_argument("masked_ref_genome",
                          help="output masked genome", type=str)
    parser_a.add_argument("-hm", "--hard_mask", help="use hard mask (defaults will running soft mask",
                          action='store_true')

    # argparse for MaskFasta2Gff
    parser_a = subparsers.add_parser('MaskFasta2Gff',
                                     help='get gff file from masked genome fasta file')

    parser_a.add_argument(
        "masked_file", help="masked gff file, N should upper", type=str)
    parser_a.add_argument(
        "output_gff", help="output_gff_file", type=str)

    # argparse for RepeatMasker2Gff3
    parser_a = subparsers.add_parser('RepeatMasker2Gff3',
                                     help='convert repeatmasker out file to gff3')

    parser_a.add_argument(
        "repeatmasker_out", help="repeatmasker out file", type=str)
    parser_a.add_argument(
        "output_gff", help="output_gff_file", type=str)

    # argparse for GeneBank2Gff3
    parser_a = subparsers.add_parser('GeneBank2Gff3',
                                     help='convert GeneBank file to gff3 and fasta file')

    parser_a.add_argument(
        "GeneBank_file", help="genebank file", type=str)
    parser_a.add_argument(
        "output_gff3", help="output_gff3", type=str)

    # argparse for Wise2EVM
    parser_a = subparsers.add_parser('Wise2EVM',
                                     help='convert genewise output gff3 (from xyx output) to EVM need gff3')

    parser_a.add_argument(
        "genewise_gff3", help="path of genewise gff3", type=str)
    parser_a.add_argument(
        "evm_gff3_output", help="path of evm output gff3", type=str)

    # argparse for ProteinByPfam
    parser_a = subparsers.add_parser('ProteinByPfam',
                                     help='give me a genome seq, by hmmscan to get protein seq, dont care stop and frameshift\n',
                                     description='')

    parser_a.add_argument('genome_seq_file', type=str, help='raw genome fasta file')
    parser_a.add_argument("pfam_db_file", help="give me pfam db", type=str)
    parser_a.add_argument("-o", "--output_file", help="output file name, default: *.pfam.aa")
    parser_a.add_argument("-w", "--tmp_work_dir", help="temp work dir, default: ./tmp")
    parser_a.add_argument("-c", "--clean_flag",
                          help="clean temp work dir", action='store_true')
    parser_a.add_argument("-t", "--threads", default=56, type=int)


    # argparse for PseudoGene
    parser_a = subparsers.add_parser('PseudoGene',
                                     help='Get pseudogene from WPGmapper results')
    parser_a.add_argument("WPGmapper_dir",
                          help="Path of WPGmapper dir",
                          type=str)
    parser_a.add_argument("reference_genome_table",
                          help="Path of reference genome table in excel format, must have column \"id\" and \"pt_file\"",
                          type=str)
    parser_a.add_argument("target_genome_fasta",
                          help="Path of target genome fasta file", type=str)
    parser_a.add_argument("known_gene_gff",
                          help="Path of known gene gff file", type=str)
    parser_a.add_argument("OG_tsv_file",
                          help="Path of orthofinder orthogroup tsv file", type=str)
    parser_a.add_argument("output_prefix",
                          help="Path of output prefix", type=str)

    parser_a.add_argument("-c", "--min_cover", help="A valid evidence alignment region needs to account for at least min_cover of the reference gene default=0.5",
                          default=0.5, type=float)
    parser_a.add_argument("-l", "--min_aa_len", help="A valid evidence alignment region needs at least min_aa_len in reference gene default=50",
                          default=50, type=int)
    parser_a.add_argument("-i", "--min_identity", help="A valid evidence alignment region needs at least min_identity in genewise indentity default=0.2",
                          default=0.2, type=float)
    parser_a.add_argument("-e", "--evalue", help="A valid evidence alignment region needs at least max evalue in blastp default=1e-10",
                          default=1e-10, type=float)
    parser_a.add_argument("-s", "--min_score", help="A valid evidence alignment region needs at least min_score in genewise score default=50",
                          default=50, type=int)
    parser_a.add_argument("-og", "--min_OG_support", help="A valid evidence alignment region needs at least query sequence have min_OG_support orthology from orthofinder default=2",
                          default=2, type=int)
    parser_a.add_argument("-cl", "--min_cluster_supp", help="A valid evidence alignment region needs at least min_cluster_supp in jaccord cluster default=2",
                          default=2, type=int)
    parser_a.add_argument("-t", "--threads", default=56, type=int)

    # argparse for AnnotationMerge
    parser_a = subparsers.add_parser('AnnotationMerge',
                                     help='merge function annotation from interpro, nr et al',
                                     description='merge function annotation from interpro, nr')

    parser_a.add_argument('protein_name_list', type=str,
                          help='Path of protein name list')
    parser_a.add_argument("-i", "--interproscan_output_tsv",
                          help="output from: ./interproscan.sh -i Cuscuta/100.seq -f tsv -b Cuscuta/100.seq.out -iprlookup --goterms --pathways -dp",
                          type=str)
    parser_a.add_argument("-n", "--nr_output_xml",
                          help="output from: diamond blastp -q gene_in_case.seq -k 50 -d ~/Database/NCBI/nr/nr -o gene_in_case.nr.bls -f 5 -p 15",
                          type=str)
    parser_a.add_argument(
        "-o", "--output_file", help="the output file (default:phyout)", default=None, type=str)

    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # command detail

    # MakerPlantRepeat
    if args_dict["subcommand_name"] == "MakerPlantRepeat":
        from api.xuyuxing.genome.maker_p import maker_p_repeat_main
        from lib.common.util import configure_parser

        # configure file argument parse

        """
        class abc():
            pass

        args = abc()
        args.genome = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker/repeat2/Gel.genome.v1.0.fasta'
        args.tag = 'Gel'
        args.config_file = None
        args.output_dir = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker/repeat2'
        """

        try:
            script_dir_path = os.path.split(os.path.realpath(__file__))[0]
            defaults_config_file = script_dir_path + \
                "/api/xuyuxing/config_file/Maker_p_defaults.ini"
        except:
            defaults_config_file = "/lustre/home/xuyuxing/python_project/Genome_work_tools/api/xuyuxing/config_file/Maker_p_defaults.ini"
        # print(defaults_config_file)

        args_type = {

            # Parameter
            "num_threads": "int",

            # Paths
            "genome": "str",
            "tag": "str",
            "trna_file": "str",
            "crl_dir": "str",
            "tpase_dir": "str",
            "pe_dir": "str",
            "old_blastx": "str",
            "protein_db": "str"

        }

        args = configure_parser(
            args, defaults_config_file, args.config_file, args_type, None)

        # Modify_dict parse
        cfg = configparser.ConfigParser()
        if args.config_file is not None:
            cfg.read(args.config_file)

        # if "Modify_dict" in cfg.sections():
        #     args.modify_dict = {}
        #     for key in cfg["Modify_dict"]:
        #         value = cfg["Modify_dict"][key]
        #         args.modify_dict[key] = value

        # output dir
        if args.output_dir is None:
            args.output_dir = os.getcwd()

        maker_p_repeat_main(args)

    # GeneStruComp
    elif args_dict["subcommand_name"] == "GeneStruComp":
        pass

    # MapOldAnno
    elif args_dict["subcommand_name"] == "MapOldAnno":

        """
        blat Gel.genome.v1.0.fasta /dev/null /dev/null -tileSize=11 -makeOoc=11.ooc -repMatch=1024

        blat Gel.genome.v1.0.fasta Gel.yuan.cDNA.fa -q=rna -dots=100  -maxIntron=500000 -out=psl -ooc=11.ooc Gel.yuan.cDNA.psl
        blat Gel.genome.v1.0.fasta Gel.yuan.CDS.fa -q=rna -dots=100  -maxIntron=500000 -out=psl -ooc=11.ooc Gel.yuan.CDS.psl

        blat2gff.pl < Gel.yuan.cDNA.psl > Gel.yuan.cDNA.gff
        blat2gff.pl < Gel.yuan.CDS.psl > Gel.yuan.CDS.gff

        class abc():
            pass

        args = abc()
        args.cDNA_gff = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/Gel.yuan.cDNA.gff"
        args.cDNA_fasta = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/Gel.yuan.cDNA.fa"
        args.old_gff = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/GWHAAEX00000000.rename.gff"
        args.old_genome = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/GWHAAEX00000000.genome.fasta"
        args.new_genome = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/Gel.genome.v1.0.fasta"
        args.tmp_dir = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/tmp"
        args.num_threads = 10
        args.output_gff = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/base_on_yuan/blat.output.gff"
        """

        from src.xuyuxing.GenomeTools.Gene2Genome import cDNA2Genome_main
        cDNA2Genome_main(args)

    # MergeMaker
    elif args_dict["subcommand_name"] == "MergeMaker":
        """
        class abc():
            pass

        args = abc()
        args.maker_gff1 = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/Gel.frag.1.maker.output/Gel.frag.1.all.noseq.gff"
        args.maker_gff2 = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/Gel.frag.2.maker.output/Gel.frag.2.all.noseq.gff"
        args.ref_genome = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/Gel.genome.v1.0.fasta"
        args.tmp_dir = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/merge_tmp"
        args.results_gff = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/Gel.all.noseq.results.gff"
        args.evidence_gff = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/Gel.all.noseq.evidence.gff"
        args.num_threads = 10
        """

        import re
        from lib.common.genome.seq_base import read_fasta_by_faidx
        from lib.common.genome.genome_feature2 import read_gff_file, ChrLoci, GenomeFeature, cluster_of_chr_loci, write_gff_file
        from lib.xuyuxing.base.base_function import multiprocess_running, mkdir

        def fancy_name_parse(input_string):
            match_obj = re.match(r'^(\S+)_(\d+)-(\d+)$', input_string)
            contig_name, c_start, c_end = match_obj.groups()

            start = min(int(c_start), int(c_end))
            end = max(int(c_start), int(c_end))

            return contig_name, start, end

        def extract_line(input_file, output_prefix, contig_list):
            """
            input_file = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/Gel.frag.1.maker.output/Gel.frag.1.all.noseq.gff'
            output_prefix = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/merge_tmp/maker1_'
            contig_list = chr_list
            """

            # open contig file
            out_handle = {}
            out_file = {}
            for contig in contig_list:
                out_file[contig] = output_prefix + "%s.gff" % contig
                out_handle[contig] = open(out_file[contig], "w")

            # split write
            with open(input_file, 'r') as f:
                for each_line in f:
                    if re.match(r'^#', each_line):
                        continue
                    each_line = re.sub('\n', '', each_line)
                    info = each_line.split("\t")
                    if len(info) > 0:
                        match_list = re.findall(r'^(\S+)_\d+-\d+$', info[0])
                        if len(match_list) > 0:
                            contig = match_list[0]
                            out_handle[contig].write(each_line + "\n")

            # close handle
            for contig in contig_list:
                out_handle[contig].close()

            return out_file

        def coord_shift(give_site, base_site):
            return base_site + give_site - 1

        def feature_coord_shift(feature):
            """
            shift feature coord with contig name
            """
            c_name, c_start, c_end = fancy_name_parse(feature.chr_id)

            new_cl = ChrLoci(chr_id=c_name, strand=feature.strand, start=coord_shift(feature.start, c_start),
                             end=coord_shift(feature.end, c_start))

            new_gf = GenomeFeature(
                id=feature.id, type=feature.type, chr_loci=new_cl)
            new_gf.qualifiers = feature.qualifiers
            new_gf.old_chr_id = feature.chr_id

            # parse sub_feature
            if hasattr(feature, 'sub_features') and (not feature.sub_features is None) and len(
                    feature.sub_features) != 0:
                new_gf.sub_features = []
                for sub_sf in feature.sub_features:
                    new_gf.sub_features.append(feature_coord_shift(sub_sf))

            return new_gf

        def read_maker_gff(maker_gff_file):
            """
            maker_gff_file = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/merge_tmp/maker2_G031N.gff'
            """
            feature_dict = read_gff_file(maker_gff_file)
            for type_tmp in feature_dict:
                for gf_id in feature_dict[type_tmp]:
                    gf = feature_dict[type_tmp][gf_id]
                    gf_coord_shift = feature_coord_shift(gf)
                    feature_dict[type_tmp][gf_id] = gf_coord_shift

            # change type class to source class
            new_feature_dict = {}
            for type_tmp in feature_dict:
                for gf_id in feature_dict[type_tmp]:
                    gf = feature_dict[type_tmp][gf_id]
                    if 'source' not in gf.qualifiers and gf.type == 'contig':
                        continue
                    source_tmp = gf.qualifiers['source'][0]
                    if source_tmp not in new_feature_dict:
                        new_feature_dict[source_tmp] = {}
                    new_feature_dict[source_tmp][gf_id] = gf

            return new_feature_dict

        def distance_between_boundary_and_cluster(cluster_range, contig_range_fancy_name):
            contig_name, start, end = fancy_name_parse(contig_range_fancy_name)
            mid_site = (cluster_range[1] + cluster_range[0]) / 2
            return min(abs(start - mid_site), abs(end - mid_site))

        def merge_maker_gff(gff1_file, gff2_file):
            """
            gff1_file = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/merge_tmp/maker1_G031N.gff'
            gff2_file = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/merge_tmp/maker2_G031N.gff'
            """

            # read gff file and change coord
            # print("read gff file")
            gff1 = read_maker_gff(gff1_file)
            gff2 = read_maker_gff(gff2_file)

            results_source = ['maker', 'augustus_masked',
                              'snap_masked', 'repeatmasker', 'repeatrunner']

            # print("merge results feature: ")
            results_feature_dict = {'maker': {}, 'augustus_masked': {}, 'snap_masked': {}, 'repeatmasker': {},
                                    'repeatrunner': {}}
            for source_tmp in results_source:
                # print(source_tmp)

                huge_gf_list = []

                if source_tmp in gff1:
                    for gf_id in gff1[source_tmp]:
                        huge_gf_list.append(gff1[source_tmp][gf_id])

                if source_tmp in gff2:
                    for gf_id in gff2[source_tmp]:
                        huge_gf_list.append(gff2[source_tmp][gf_id])

                cluster_dict = cluster_of_chr_loci(
                    huge_gf_list, overlap_threshold=0.5)

                for contig_id in cluster_dict:
                    for strand in cluster_dict[contig_id]:
                        for group_id in cluster_dict[contig_id][strand]:
                            cl_range = cluster_dict[contig_id][strand][group_id]['range'][0]
                            gf_in_range = cluster_dict[contig_id][strand][group_id]['list']
                            good_gf = \
                                sorted(gf_in_range,
                                       key=lambda x: distance_between_boundary_and_cluster(
                                           cl_range, x.old_chr_id),
                                       reverse=True)[0]
                            results_feature_dict[source_tmp][good_gf.id] = good_gf

            evidence_source = ['blastn', 'blastx',
                               'est2genome', 'protein2genome']

            # print("merge results feature: ")
            evidence_feature_dict = {'blastn': {}, 'blastx': {
            }, 'est2genome': {}, 'protein2genome': {}}
            for source_tmp in evidence_source:
                # print(source_tmp)

                # grouped by Name
                name_dict = {}

                if source_tmp in gff1:
                    for gf_id in gff1[source_tmp]:
                        gf = gff1[source_tmp][gf_id]
                        evidence_name = gf.qualifiers['Name'][0]
                        if not evidence_name in name_dict:
                            name_dict[evidence_name] = []
                        name_dict[evidence_name].append(gf)

                if source_tmp in gff2:
                    for gf_id in gff2[source_tmp]:
                        gf = gff2[source_tmp][gf_id]
                        evidence_name = gf.qualifiers['Name'][0]
                        if not evidence_name in name_dict:
                            name_dict[evidence_name] = []
                        name_dict[evidence_name].append(gf)

                for evidence_name in name_dict:
                    gf_list = name_dict[evidence_name]
                    cluster_dict = cluster_of_chr_loci(
                        gf_list, overlap_threshold=0.5)

                    for contig_id in cluster_dict:
                        for strand in cluster_dict[contig_id]:
                            for group_id in cluster_dict[contig_id][strand]:
                                cl_range = cluster_dict[contig_id][strand][group_id]['range'][0]
                                gf_in_range = cluster_dict[contig_id][strand][group_id]['list']
                                good_gf = \
                                    sorted(gf_in_range,
                                           key=lambda x: distance_between_boundary_and_cluster(
                                               cl_range, x.old_chr_id),
                                           reverse=True)[0]
                                evidence_feature_dict[source_tmp][good_gf.id] = good_gf

            return results_feature_dict, evidence_feature_dict

        if args.tmp_dir is None:
            args.tmp_dir = os.getcwd() + '/tmp'

        mkdir(args.tmp_dir, True)
        print("tmp dir is : %s" % args.tmp_dir)

        ref_dict = read_fasta_by_faidx(args.ref_genome)
        chr_list = list(ref_dict.keys())

        maker_out = {
            "ID1": {'raw_file': args.maker_gff1, 'split_gff_prefix': args.tmp_dir + "/maker1_"},
            "ID2": {'raw_file': args.maker_gff2, 'split_gff_prefix': args.tmp_dir + "/maker2_"},
        }

        # extract contig line in a tmp file to read
        for id in maker_out:
            maker_out[id]['gff'] = extract_line(
                maker_out[id]['raw_file'], maker_out[id]['split_gff_prefix'], chr_list)

        # merge anno for each contig
        args_list = []
        for contig in chr_list:
            # print(contig)

            gff1_file = maker_out['ID1']['gff'][contig]
            gff2_file = maker_out['ID2']['gff'][contig]

            # results_feature_dict, evidence_feature_dict = merge_maker_gff(gff1_file, gff2_file)
            args_list.append((gff1_file, gff2_file))

        merge_output = multiprocess_running(
            merge_maker_gff, args_list, args.num_threads)

        huge_results_feature_list = []
        huge_evidence_feature_list = []

        for i in merge_output:

            results_feature_dict, evidence_feature_dict = merge_output[i]['output']

            for source_tmp in results_feature_dict:
                for gf_id in results_feature_dict[source_tmp]:
                    gf = results_feature_dict[source_tmp][gf_id]
                    huge_results_feature_list.append(gf)

            for source_tmp in evidence_feature_dict:
                for gf_id in evidence_feature_dict[source_tmp]:
                    gf = evidence_feature_dict[source_tmp][gf_id]
                    huge_evidence_feature_list.append(gf)

        write_gff_file(huge_results_feature_list, args.results_gff)
        write_gff_file(huge_evidence_feature_list, args.evidence_gff)

    # Wise2EVM
    elif args_dict["subcommand_name"] == "Wise2EVM":
        """
        class abc():
            pass

        args = abc()
        args.genewise_gff3 = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/pasa/protein/genewise.gff3"
        args.evm_gff3_output = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/pasa/protein/genewise.evm.gff3"
        """

        from lib.common.genome.genome_feature2 import read_gff_file
        from lib.common.util import printer_list

        gf_dict = read_gff_file(args.genewise_gff3)

        with open(args.evm_gff3_output, 'w') as f:
            for source_tmp in gf_dict:
                for gf_id in gf_dict[source_tmp]:
                    gf = gf_dict[source_tmp][gf_id]
                    mRNA = gf.sub_features[0]
                    target = mRNA.qualifiers['Target'][0]
                    cds_list = mRNA.sub_features

                    cds_list = sorted(cds_list, key=lambda x: int(
                        x.qualifiers['Target_Start'][0]))

                    for cds in cds_list:
                        target_start = int(cds.qualifiers['Target_Start'][0])
                        target_end = int(cds.qualifiers['Target_End'][0])

                        printer_string = printer_list(
                            [mRNA.chr_id, 'genewise', 'match', cds.start, cds.end, '.', cds.strand, '.',
                             'ID=%s;Target=%s %d %d' % (mRNA.id, target, target_start, target_end)])

                        f.write(printer_string + "\n")

    # MaskFromMaker
    elif args_dict["subcommand_name"] == "MaskFromMaker":
        """
        class abc():
            pass

        args = abc()
        args.maker_gff = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/Gel.results.gff"
        args.ref_genome = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/Gel.genome.v1.0.fasta"
        args.masked_ref_genome = "/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker_v2/round1/Gel.genome.v1.0.masked.fasta"
        args.hard_mask = False
        """

        from lib.common.genome.genome_feature2 import read_gff_file
        from lib.common.genome.seq_base import read_fasta_by_faidx, BioSeq
        from lib.xuyuxing.math.set_operating import merge_intervals, interval_minus_set

        def get_seq(seq, get_range, mask):
            if mask == 'hard':
                return 'N' * (abs(get_range[1] - get_range[0]) + 1)
            elif mask == 'soft':
                return seq[get_range[0] - 1:get_range[1]].lower()
            else:
                return seq[get_range[0] - 1:get_range[1]].upper()

        def seq_masker(seq, mask_range_list, hard_mask=False):
            """
            seq = 'ATGCATGCATCGTA'
            mask_range_list = [(1,14)]
            hard_mask = False
            """
            if len(mask_range_list) == 0:
                return seq

            all_range = (1, len(seq))
            mask_range_list = merge_intervals(mask_range_list, True)
            unmask_range_list = interval_minus_set(all_range, mask_range_list)
            unmask_range_list = merge_intervals(unmask_range_list, True)

            if hard_mask:
                mask = 'hard'
            else:
                mask = 'soft'

            if len(mask_range_list) == 1 and mask_range_list[0] == all_range:
                return get_seq(seq, mask_range_list[0], mask)

            output_seq = ""

            if mask_range_list[0][0] < unmask_range_list[0][1]:
                for i in range(len(mask_range_list)):
                    output_seq += get_seq(seq, mask_range_list[i], mask)
                    if i <= len(unmask_range_list) - 1:
                        output_seq += get_seq(seq,
                                              unmask_range_list[i], 'no mask')
            else:
                for i in range(len(unmask_range_list)):
                    output_seq += get_seq(seq, unmask_range_list[i], 'no mask')
                    if i <= len(mask_range_list) - 1:
                        output_seq += get_seq(seq, mask_range_list[i], mask)

            return output_seq

        gf_dict = read_gff_file(args.maker_gff)
        genome_dict = read_fasta_by_faidx(args.ref_genome)

        repeat_source = ['repeatmasker', 'repeatrunner']

        repeat_gf_by_contig = {}
        for source_tmp in repeat_source:
            for type_tmp in gf_dict:
                for gf_id in gf_dict[type_tmp]:
                    gf = gf_dict[type_tmp][gf_id]
                    source_tmp = gf.qualifiers['source'][0]
                    if source_tmp not in repeat_source:
                        continue
                    if gf.chr_id not in repeat_gf_by_contig:
                        repeat_gf_by_contig[gf.chr_id] = {}
                    repeat_gf_by_contig[gf.chr_id][gf.id] = gf

        # todo make a repeat genus report

        contig_mask_range_dict = {}
        for contig_id in repeat_gf_by_contig:
            range_list = [repeat_gf_by_contig[contig_id]
                          [gf_id].range for gf_id in repeat_gf_by_contig[contig_id]]
            range_list = merge_intervals(range_list, True)
            contig_mask_range_dict[contig_id] = range_list

        mask_site = 0
        for contig_id in contig_mask_range_dict:
            for range_tmp in contig_mask_range_dict[contig_id]:
                mask_site += abs(range_tmp[1] - range_tmp[0]) + 1

        total_len = 0
        for contig_id in genome_dict:
            contig_seq = genome_dict[contig_id]
            total_len += contig_seq.len()

        print("%d in %d bp (%.2f%%) will be masked" %
              (mask_site, total_len, mask_site / total_len * 100))

        for contig_id in sorted(list(genome_dict.keys())):
            contig_seq = genome_dict[contig_id]
            seq_str = contig_seq.seq.upper()

            if contig_id in contig_mask_range_dict:
                seq_str = seq_masker(
                    seq_str, contig_mask_range_dict[contig_id], hard_mask=args.hard_mask)

            new_contig_seq = BioSeq(seq=seq_str, seqname=contig_seq.seqname)
            # new_contig_seq.wrap()
            new_contig_seq.write_to_file(args.masked_ref_genome)

    # AnnotationMerge
    elif args_dict["subcommand_name"] == "AnnotationMerge":

        import sys
        from api.xuyuxing.file_parser.interproscan_results import interproscan_results_parser
        from api.common.genome.blast import NR_blast_annotation_extractor
        from lib.common.fileIO import tsv_file_dict_parse
        from lib.common.util import printer_list

        """
        class abc():
            pass
        
        args = abc()
        args.protein_name_list = "/lustre/home/xuyuxing/Work/Other/songjuan/gene.list"
        args.interproscan_output_tsv = "/lustre/home/xuyuxing/Work/Other/songjuan/interproscan.out.tsv"
        args.nr_output_xml = "/lustre/home/xuyuxing/Work/Other/songjuan/nr.bls"
        args.output_file = "/lustre/home/xuyuxing/Work/Other/songjuan/gene.anno.txt"
        """

        protein_name_list = args.protein_name_list
        interproscan_output_tsv = args.interproscan_output_tsv
        nr_output_xml = args.nr_output_xml
        output_file = args.output_file

        if not output_file is None:
            F1 = open(output_file, "w")
            sys.stdout = F1

        protein_name_list = list(tsv_file_dict_parse(
            protein_name_list, key_col="ID", fieldnames=["ID"]).keys())
        interpro_dict = interproscan_results_parser(interproscan_output_tsv)
        nr_dict = NR_blast_annotation_extractor(nr_output_xml)

        for i in protein_name_list:
            if i in interpro_dict:
                interpro_anno = [
                    j["Description"] for j in interpro_dict[i].domain_list if j["Analysis"] == "Pfam"]
                interpro_anno = list(set(interpro_anno))
                interpro_anno_txt = printer_list(interpro_anno, ";", "")
                go_anno = []
                for j in interpro_dict[i].domain_list:
                    if not j["GO"] is None and not j['GO'] == '':
                        go_anno.extend(j["GO"].split("|"))
                go_anno = list(set(go_anno))
                go_anno_txt = printer_list(go_anno, ";", "")
            else:
                interpro_anno_txt = ""
            if i in nr_dict:
                nr_anno = nr_dict[i]
                nr_anno = list(set(nr_anno))
                nr_anno_txt = printer_list(nr_anno, ";", "")
            else:
                nr_anno_txt = ""

            print("%s\t%s\t%s\t%s" %
                  (i, interpro_anno_txt, go_anno_txt, nr_anno_txt))

        if not output_file is None:
            F1.close()

    # PseudoGene
    elif args_dict["subcommand_name"] == "PseudoGene":
        from src.xuyuxing.GenomeTools.PseudoGene import pseudogene_main
        pseudogene_main(args)

    # AnnoCleaner
    elif args_dict["subcommand_name"] == "AnnoCleaner":
        from src.xuyuxing.GenomeTools.AnnoCleaner import AnnoCleaner_main
        AnnoCleaner_main(args)

    # EasyMaker
    elif args_dict["subcommand_name"] == "EasyMaker":
        from lib.common.util import configure_parser
        from api.xuyuxing.genome.maker import maker_main

        # configure file argument parse

        """
        class abc():
            pass

        args = abc()
        args.genome = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker/repeat2/Gel.genome.v1.0.fasta'
        args.tag = 'Gel'
        args.config_file = None
        args.output_dir = '/lustre/home/xuyuxing/Database/Gel/genome/annotation/maker/repeat2'
        """

        try:
            script_dir_path = os.path.split(os.path.realpath(__file__))[0]
            defaults_config_file = script_dir_path + \
                "/api/xuyuxing/config_file/Maker_defaults.ini"
        except:
            defaults_config_file = "/lustre/home/xuyuxing/python_project/Genome_work_tools/api/xuyuxing/config_file/Maker_defaults.ini"
        # print(defaults_config_file)

        args_type = {

            # Parameter
            "num_threads": "int",

            # Paths
            "tag": "str",
            "genome_file": "str",
            "repeat_file": "str",
            "est_file": "str",
            "protein_file": "str",
            "rrna_file": "str",
            "augustus_species": "str",
            "repeat_protein": "str",
            "work_dir": "str",
            "est_gff": "str",
            "protein_gff": "str",
            "rm_gff": "str",
            "snaphmm": "str",
        }

        args = configure_parser(
            args, defaults_config_file, args.config_file, args_type, None)

        # Modify_dict parse
        cfg = configparser.ConfigParser()
        if args.config_file is not None:
            cfg.read(args.config_file)

        # if "Modify_dict" in cfg.sections():
        #     args.modify_dict = {}
        #     for key in cfg["Modify_dict"]:
        #         value = cfg["Modify_dict"][key]
        #         args.modify_dict[key] = value

        args.work_dir = os.path.abspath(args.work_dir)

        # print(vars(args))
        maker_main(args)

    # Protein2Genome
    elif args_dict["subcommand_name"] == "Protein2Genome":
        from src.xuyuxing.GenomeTools.Gene2Genome import Protein2Genome_main
        Protein2Genome_main(args)

    # MaskFasta2Gff
    elif args_dict["subcommand_name"] == "MaskFasta2Gff":
        from src.xuyuxing.GenomeTools.MaskFasta2gff import MaskFasta2Gff_main
        MaskFasta2Gff_main(args)

    # RepeatMasker2Gff3
    elif args_dict["subcommand_name"] == "RepeatMasker2Gff3":
        from src.xuyuxing.GenomeTools.RepeatMasker2Gff3 import RepeatMasker2Gff3_main
        RepeatMasker2Gff3_main(args)

    # GeneBank2Gff3
    elif args_dict["subcommand_name"] == "GeneBank2Gff3":
        from src.xuyuxing.GenomeTools.GeneBank2Gff3 import GeneBank2Gff3_main
        GeneBank2Gff3_main(args)

    # GeneBank2Gff3
    elif args_dict["subcommand_name"] == "ProteinByPfam":
        from src.xuyuxing.GenomeTools.ProteinByPfam import ProteinByPfam_main
        ProteinByPfam_main(args)


from Bio import Phylo
from lib.common.evolution.tree_operate import add_clade_name

tree_file = "/Users/yuxingxu/Job/Work/HGT/treeplot/tree.phb"
tree = Phylo.read(tree_file, 'newick')
tree = add_clade_name(tree)

tree_dir = {}
for clade in tree.find_clades():
    tree_dir[clade.name] = clade

clade_N6 = tree_dir['N6']
clade_N7 = tree_dir['N7']

from Bio.Phylo.BaseTree import Tree, Clade


class CladePlot(object):

    def __init__(self, Clade, **kwargs):
        self.clade = Clade
        self.input_setting = kwargs

        # change attr in clade
        for k, v in self.input_setting:
            if k == 'color':
                self.clade = v

def is_offspring_of(A, B):
    """
    A is offspring of B
    A can not be son of B
    """
    return B in A.get_ancestors()








def tree_plot


class abc(object):
    def __init__(self, a, b, c):
        self.a = a
        self.b = b
        self.c = c

    def print_abc(self):
        print(self.a, self.b, self.c)


class abc_with_d(abc):
    def __init__(self, abc, d):
        self = abc
        self.d = d

    def print_abcd(self):
        print(self.a, self.b, self.c, self.d)


def add_d(abc, d):


ABC = abc(1, 2, 3)
ABCD = abc_with_d(ABC, 4)



from lib.common.os import mkdir, copy_file, cmd_run, multiprocess_running
from lib.common.util import logging_init
from lib.xuyuxing.evolution.orthotools import OG_tsv_file_parse
import os
import pandas as pd
from lib.common.evolution.tree_operate import get_root_by_species, add_clade_name, lookup_by_names, reroot_by_outgroup_clade, collapse_low_support
from Bio import Phylo
from lib.common.fileIO import tsv_file_dict_parse, excel_file_parse
from lib.common.math.interval import merge_intervals, overlap_between_interval_set, \
    group_by_intervals_with_overlap_threshold, sum_interval_length
import pickle
from collections import OrderedDict
from api.xuyuxing.genome.genblasta import genblasta_run, genblasta_to_genewise
from lib.common.genome.seq_base import read_fasta_by_faidx, read_fasta
from config import wu_blast_path
from BCBio import GFF
from lib.common.genome.genome_feature2 import read_gff_file, sub_gf_traveler
from interlap import InterLap
from api.common.genome.blast import outfmt6_read_big, evalue_to_wvalue
from lib.xuyuxing.base.common_command import merge_dict
from lib.common.evolution.tree_operate import monophyly
from src.xuyuxing.tools.phytools import PtTree


def if_conserved(species_list, conserved_arguments_dict):
    core_ref_list = conserved_arguments_dict["core_ref_list"]
    norm_ref_list = conserved_arguments_dict["norm_ref_list"]
    min_genome_in_norm_ref_tolerance = conserved_arguments_dict["min_genome_in_norm_ref_tolerance"]

    conserved_flag = True

    if len(set(species_list) & set(core_ref_list)) != len(core_ref_list):
        conserved_flag = False

    if len(set(species_list) & set(norm_ref_list)) < min_genome_in_norm_ref_tolerance:
        conserved_flag = False

    return conserved_flag


def if_conserved2(species_list, conserved_arguments_dict):
    """
    full_species_list = ["1", "2", "3", "4", "5", "6", "7", "8", "9"]

    core_ref_list = ["1", "2"]
    sp_group_lol = [["3", "4", "5"],["6", "7"],["8", "9"]]
    group_min_num_list = [2,1,1]

    species_list = ["1","2","3","4","6","8"]
    """

    core_ref_list = conserved_arguments_dict["core_ref_list"]
    sp_group_lol = conserved_arguments_dict["sp_group_lol"]
    group_min_num_list = conserved_arguments_dict["group_min_num_list"]

    conserved_flag = True

    if len(set(species_list) & set(core_ref_list)) != len(core_ref_list):
        conserved_flag = False

    for sp_group_list, min_num in zip(sp_group_lol, group_min_num_list):
        if len(set(species_list) & set(sp_group_list)) < min_num:
            conserved_flag = False

    return conserved_flag


def get_conserved_clades(rooted_tree_node_dict, gene_to_species_map_dict, conserved_function, conserved_arguments):
    """
    filter all conserved clades on a rooted tree, return clades may overlap with each other
    """
    conserved_clades = []
    for clade_id in rooted_tree_node_dict:
        clade = rooted_tree_node_dict[clade_id]
        if not clade.is_terminal():
            sp_list = list(set([gene_to_species_map_dict[leaf.name]
                                for leaf in clade.get_terminals()]))
            if conserved_function(sp_list, conserved_arguments):
                conserved_clades.append(clade_id)
    return conserved_clades


def get_orthogroups_from_tree(tree_prefix, gene_tree, species_tree, gene_to_species_map_dict, conserved_function, conserved_arguments, support_threshold):
    # root gene tree
    gene_tree = add_clade_name(gene_tree)
    gene_tree_node_dict = lookup_by_names(gene_tree)

    best_root_clade = get_root_by_species(
        gene_tree, species_tree, gene_to_species_map_dict)

    gene_tree_rooted, gene_tree_rooted_node_dict, gene_tree, gene_tree_node_dict = reroot_by_outgroup_clade(gene_tree,
                                                                                                            gene_tree_node_dict,
                                                                                                            best_root_clade.name,
                                                                                                            True)

    # collapse low support clades
    gene_tree_rooted_collapsed = collapse_low_support(
        gene_tree_rooted, support_threshold)
    gene_tree_rooted_collapsed = add_clade_name(gene_tree_rooted_collapsed)
    gene_tree_rooted_collapsed_node_dict = lookup_by_names(
        gene_tree_rooted_collapsed)

    # get conserved clades
    conserved_clades_id_list = get_conserved_clades(
        gene_tree_rooted_collapsed_node_dict, gene_to_species_map_dict, conserved_function, conserved_arguments)

    # get sub_orthogroups
    num = 0

    orthogroup_id = "%s_%d" % (tree_prefix, num)
    zero_clade_conserved = conserved_function(list(set(
        [gene_to_species_map_dict[leaf.name] for leaf in gene_tree_rooted_collapsed.get_terminals()])), conserved_arguments)
    sub_orthogroup_dict = {orthogroup_id: (
        [leaf.name for leaf in gene_tree_rooted_collapsed.get_terminals()], zero_clade_conserved)}

    for clade_id in conserved_clades_id_list:
        num += 1
        orthogroup_id = "%s_%d" % (tree_prefix, num)
        clade = gene_tree_rooted_collapsed_node_dict[clade_id]
        sub_orthogroup_dict[orthogroup_id] = (
            [leaf.name for leaf in clade.get_terminals()], True)

    return sub_orthogroup_dict


def get_orthopairs_from_orthogroups(sp_a, sp_b, orthogroup_dict, gene_to_species_map_dict):
    sp_a_dict = {}

    for og_id in orthogroup_dict:
        orthogroup_gene_list = orthogroup_dict[og_id][0]

        a_list = tuple(
            [gene_id for gene_id in orthogroup_gene_list if gene_to_species_map_dict[gene_id] == sp_a])
        b_list = tuple(
            [gene_id for gene_id in orthogroup_gene_list if gene_to_species_map_dict[gene_id] == sp_b])

        for a in a_list:
            if a not in sp_a_dict:
                sp_a_dict[a] = (b_list, og_id)
            else:
                if len(b_list) < len(sp_a_dict[a]):
                    sp_a_dict[a] = (b_list, og_id)

    return sp_a_dict


def get_orthopairs_from_one_tree_dir(sp_a, sp_b, gene_tree_file, species_tree_file, gene_to_species_map_file, conserved_function, conserved_arguments, support_threshold):
    gene_tree = Phylo.read(gene_tree_file, 'newick')
    species_tree = Phylo.read(species_tree_file, 'newick')

    info_dict = tsv_file_dict_parse(gene_to_species_map_file, fieldnames=[
                                    "new_id", "old_id", "sp_id"])
    gene_to_species_map_dict = {}
    rename_dict = {}
    for i in info_dict:
        gene_to_species_map_dict[info_dict[i]["new_id"] +
                                 "_" + info_dict[i]["sp_id"]] = info_dict[i]["sp_id"]
        rename_dict[info_dict[i]["new_id"] + "_" +
                    info_dict[i]["sp_id"]] = info_dict[i]["old_id"]

    sub_orthogroup_dict = get_orthogroups_from_tree(
        "text", gene_tree, species_tree, gene_to_species_map_dict, conserved_function, conserved_arguments, support_threshold=support_threshold)

    ortho_output_dict = get_orthopairs_from_orthogroups(
        sp_a, sp_b, sub_orthogroup_dict, gene_to_species_map_dict)

    output_dict = {}
    for a_g_id in ortho_output_dict:
        b_g_id_tuple, support_OG_id = ortho_output_dict[a_g_id]
        support_OG_id_list = sub_orthogroup_dict[support_OG_id][0]
        old_a_g_id = rename_dict[a_g_id]
        old_b_g_id_tuple = tuple([rename_dict[i] for i in b_g_id_tuple])
        old_support_OG_id_list = tuple(
            [rename_dict[i] for i in support_OG_id_list])

        output_dict[old_a_g_id] = (
            old_b_g_id_tuple, old_support_OG_id_list, sub_orthogroup_dict[support_OG_id][1])

    return output_dict


def get_orthopairs_fasttree_pipeline(OG_tsv_file, fasttree_dir, species_tree_file, sp_a, sp_b, conserved_function, conserved_arguments, support_threshold, threads=56):
    """
    output_dict['Ath_id'] = (('Gel_id',), (OG_gene_list), conserved_flag)
    """

    OG_dict = OG_tsv_file_parse(OG_tsv_file)

    num = 0

    args_list = []

    for og_id in OG_dict:
        num += 1

        og_dir = fasttree_dir + "/" + og_id
        gene_tree_file = og_dir + "/fasttree.phb"
        gene_to_species_map_file = og_dir + "/rename.map"
        args_list.append((sp_a, sp_b, gene_tree_file, species_tree_file, gene_to_species_map_file,
                          conserved_function, conserved_arguments, support_threshold))

    tmp_out = multiprocess_running(
        get_orthopairs_from_one_tree_dir, args_list, threads, None, True, None)

    output_dict = {}
    for i in tmp_out:
        tmp_output_dict = tmp_out[i]['output']
        for i in tmp_output_dict:
            output_dict[i] = tmp_output_dict[i]

    return output_dict


# gene loss test

def if_OG_conserved(species_list, conserved_arguments_dict):
    """
    full_species_list = ["1", "2", "3", "4", "5", "6", "7", "8", "9"]

    sp_group_lol = [["3", "4", "5"],["6", "7"],["8", "9"]]
    group_min_num_list = [2,1,1]

    species_list = ["1","2","3","4","6","8"]
    """
    sp_group_lol = conserved_arguments_dict["sp_group_lol"]
    group_min_num_list = conserved_arguments_dict["group_min_num_list"]

    conserved_flag = True

    for sp_group_list, min_num in zip(sp_group_lol, group_min_num_list):
        if len(set(species_list) & set(sp_group_list)) < min_num:
            conserved_flag = False

    return conserved_flag


def get_conserved_OGs(OG_dict, conserved_arguments):
    """
    filter all Orthogroups, return conserved orthogroups id list
    """
    conserved_OG_list = []
    for og_id in OG_dict:
        sp_list = [sp_id for sp_id in OG_dict[og_id]
                   if len(OG_dict[og_id][sp_id]) != 0]
        if if_OG_conserved(sp_list, conserved_arguments):
            conserved_OG_list.append(og_id)

    return conserved_OG_list


def load_annotated_range(gff_file, target_feature="CDS"):
    gf_dict = read_gff_file(gff_file)
    annotated_range_dict = {}
    for top_feature in gf_dict:
        for gf_id in gf_dict[top_feature]:
            gf = gf_dict[top_feature][gf_id]
            for sub_gf in sub_gf_traveler(gf):
                if sub_gf.type == target_feature:
                    chr_id = sub_gf.chr_id
                    strand = sub_gf.strand
                    if chr_id not in annotated_range_dict:
                        annotated_range_dict[chr_id] = {
                            "+": InterLap(),
                            "-": InterLap(),
                        }
                    annotated_range_dict[chr_id][strand].add(
                        (sub_gf.start, sub_gf.end))

    return annotated_range_dict


def load_genewise_gff(gff_file):
    output_gf_dict = {}
    gf_dict = read_gff_file(gff_file)
    for top_feature in gf_dict:
        for gf_id in gf_dict[top_feature]:
            output_gf_dict[gf_id] = gf_dict[top_feature][gf_id]
    return output_gf_dict


def tree_check(og_dir, target_sp_id, cpu_num, support_threshold, jaccard_threshold):
    seq_info_file = og_dir + "/seq_info.pyb"
    TEMP = open(seq_info_file, 'rb')
    ref_seq_dict, ref_seq_sp_map, homo_seq_dict, homo_seq_sp_map, add_seq_dict = pickle.load(
        TEMP)
    TEMP.close()

    # if no new wise out
    if len(add_seq_dict) == 0:
        return True, "No new genewise"

    # if no outgroup
    if len(homo_seq_dict) == 0:
        if len(add_seq_dict) != 0:
            return False, "No outgroup and have new genewise"

    # else need treeing
    tree_dir = og_dir + "/tree"
    mkdir(tree_dir, True)
    tree_output_dir = tree_dir + "/tree_out"
    mkdir(tree_output_dir, True)
    tree_file = tree_output_dir + "/tree_out.phb"

    seq_dict = merge_dict([ref_seq_dict, homo_seq_dict, add_seq_dict], False)
    map_dict = merge_dict([ref_seq_sp_map, homo_seq_sp_map], False)
    for i in add_seq_dict:
        map_dict[i] = target_sp_id

    # make tree
    top_tree_seq_file = tree_dir + "/tree.faa"
    with open(top_tree_seq_file, 'w') as f:
        for seq_name in seq_dict:
            f.write(">%s\n%s\n" % (seq_name, seq_dict[seq_name]))

    if not(os.path.exists(tree_file) and os.path.getsize(tree_file) != 0):
        PtTree(top_tree_seq_file, 'fasttree', False,
               tree_output_dir, cpu_num, "clustalo")

    # tree check
    query_gene_name_list = list(ref_seq_dict.keys())
    new_gene_name_list = list(add_seq_dict.keys())
    target_speci_gene_name_list = [
        i for i in homo_seq_sp_map if homo_seq_sp_map[i] == target_sp_id]

    tree = Phylo.read(tree_file, 'newick')
    good_topo, monophyly_leaf = monophyly(query_gene_name_list, tree, support_threshold=support_threshold,
                                          jaccard_threshold=jaccard_threshold, exclude_leaf_set=new_gene_name_list)

    added_new_gene = list(set(new_gene_name_list) & set(monophyly_leaf)) + list(
        set(target_speci_gene_name_list) & set(monophyly_leaf))

    if good_topo:
        if len(added_new_gene):
            return False, added_new_gene
        else:
            return True, "Good topo and no added gene"
    else:
        return False, "Bad topo"


def main(args):
    """
    class abc():
        pass

    args = abc()

    # path
    args.aa_fof_excel = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/2.order_level_OrthoFinder/Asparagales/pt_file/tmp/aa_fof.xlsx"
    args.orthogroups_tsv = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/2.order_level_OrthoFinder/Asparagales/pt_file/tmp/Orthogroups.tsv"
    args.target_genome_fasta = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/2.order_level_OrthoFinder/Asparagales/pt_file/tmp/T91201N0.genome.fasta"
    args.target_gene_gff = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/2.order_level_OrthoFinder/Asparagales/pt_file/tmp/T91201N0.genome.gff3"
    args.work_dir = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/2.order_level_OrthoFinder/Asparagales/pt_file/tmp"

    # parameter
    args.target_list = "Gel"
    args.core_ref_list = ["Aof", "Ath", "Osa"]
    args.norm_ref_list = ["Ash", "Dca", "Peq", "Vpl"]
    args.bad_genome_in_norm_ref_tolerance = 1
    args.flank_rage = 1000
    args.gene_coverage = 0.5
    args.genblasta_hit_num = 20
    args.annotated_coverage = 0.8
    args.same_loci_overlap_threshold = 0.5
    args.query_in_mcl_cluster_ratio = 0.8
    args.support_threshold = 0.8
    args.jaccard_threshold = 0.8
    args.top_outgroup_num = 5    
    """

    mkdir(args.work_dir, True)

    log_file = args.work_dir + "/log"

    logger = logging_init("GeneLoss", log_file)

    args_info_string = "Argument list:\n"
    attrs = vars(args)

    for item in attrs.items():
        args_info_string = args_info_string + ("%s: %s\n" % item)

    logger.info(args_info_string)

    logger.info("Step1: parse orthogroups tsv")
    ortho_dict = OG_tsv_file_parse(args.orthogroups_tsv)
    species_list_in_ortho = list(ortho_dict[list(ortho_dict.keys())[0]].keys())

    pt_file_df = pd.read_excel(
        args.aa_fof_excel, sheet_name='Sheet1', header=[0])
    species_list_in_pt_file = list(pt_file_df.sp_id)

    if set(species_list_in_ortho) != set(species_list_in_ortho):
        raise ValueError("species should same in orthofile and pt_file")

    sp_list = species_list_in_ortho
    ortho_count_df = pd.DataFrame(
        columns=sp_list, index=list(ortho_dict.keys()))

    for og_id in ortho_count_df.index:
        ortho_count_df.loc[og_id] = [
            len(ortho_dict[og_id][sp_id]) for sp_id in ortho_count_df.columns]

    logger.info("Step2: defined conserved OG")
    ortho_count_df.insert(ortho_count_df.shape[1], 'conserved', False)
    for og_id in ortho_count_df.index:
        # print(og_id)
        sp_list_now = [
            sp_id for sp_id in sp_list if ortho_count_df.loc[og_id, sp_id] != 0]
        ortho_count_df.loc[og_id, "conserved"] = if_conserved(
            sp_list_now, args)

    # todo

    ortho_obj = OrthoFinderResults(args.orthofinder_dir_path)

    # get file and dir info
    if not hasattr(args, 'modify_dict'):
        args.modify_dict = {}

    ortho_obj.pase_file_and_dir(modify_dict=args.modify_dict)

    # get orthologues or orthogroups
    ortho_obj.get_name_info(hash_table=True)
    if args.ortho_level == 'orthologues':
        ortho_obj.get_Orthologues()
        group_dir = ortho_obj.orthologues_dir

    elif args.ortho_level == 'orthogroups':
        ortho_obj.get_OrthoGroups()
        group_dir = ortho_obj.orthogroup_dir

    ortho_obj.read_aa_fasta()
    ortho_obj.blast_to_sqlite(threads=args.num_threads)

    # get candidate gene loss orthologues
    logger.info("Step2: get candidate gene loss orthologues")
    target_speci = [
        i for i in ortho_obj.species_info if ortho_obj.species_info[i]['id'] == args.target_speci][0]
    core_ref = [i for i in ortho_obj.species_info if ortho_obj.species_info[i]
                ['id'] in args.core_ref.split(',')]
    norm_ref = [i for i in ortho_obj.species_info if ortho_obj.species_info[i]
                ['id'] in args.norm_ref.split(',')]

    candidate_gene_loss = args.output_dir + "/1.candidate_gene_loss.txt"
    candi_loss_list = []
    conserved_OL_list = []

    if not os.path.exists(candidate_gene_loss):
        logger.info("first running Step2")
        for OL_id in group_dir:
            conserve_flag, target_loss_flag = if_conserve_and_target_speci_loss(group_dir[OL_id],
                                                                                target_speci,
                                                                                core_ref, norm_ref,
                                                                                args.tolerance_ratio)
            group_dir[OL_id].target_loss_flag = target_loss_flag
            group_dir[OL_id].conserve_flag = conserve_flag
            if target_loss_flag:
                candi_loss_list.append(OL_id)
            if conserve_flag:
                conserved_OL_list.append(OL_id)

        print_speci_list = args.core_ref.split(
            ',') + args.norm_ref.split(',') + args.target_speci.split(',')
        print_speci_id_list = []
        for i in print_speci_list:
            print_speci_id_list.append(
                [j for j in ortho_obj.species_info if ortho_obj.species_info[j]['id'] == i][0])

        with open(candidate_gene_loss, 'w') as f:
            f.write(printer_list(print_speci_list, head='OL_id\t') + "\n")
            for OL_id in conserved_OL_list:
                gene_set = group_dir[OL_id]
                speci_stat_dir = gene_set.speci_stat()
                gene_num_list = []
                for i in print_speci_id_list:
                    if i in speci_stat_dir:
                        gene_num_list.append(speci_stat_dir[i])
                    else:
                        gene_num_list.append(0)
                f.write(printer_list(gene_num_list, head=OL_id + "\t") + "\n")
    else:
        logger.info(
            "find 1.candidate_gene_loss.txt, get gene loss list from this file")
        gene_loss_dict = tsv_file_dict_parse(candidate_gene_loss)
        for i in gene_loss_dict:
            conserved_OL_list.append(gene_loss_dict[i]['OL_id'])
            if int(gene_loss_dict[i][args.target_speci]) == 0:
                candi_loss_list.append(gene_loss_dict[i]['OL_id'])


if __name__ == '__main__':

    # orthogroups level gene loss test

    OG_tsv_file = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.all_OrthoFinder/pt_file/OrthoFinder/Results_Mar16/Orthogroups/Orthogroups.tsv"
    work_dir = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.all_OrthoFinder/gene_loss_check"
    species_info_xlsx = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.all_OrthoFinder/gene_loss_check/species_info.xlsx"

    species_info_dict = excel_file_parse(species_info_xlsx, key_col='sp_id')

    conserved_arguments = {
        "sp_group_lol": [['Csi', 'Cnu', 'Acom', 'Bdi', 'Osa', 'Oth', 'Zma', 'Eve', 'Mac'], ['Vvi', 'Ptr', 'Mes', 'Gma', 'Fve', 'Csa', 'Ath', 'Gra'], ['Cca', 'Mgu', 'Oeu', 'Sin', 'Can', 'Sly', 'Stu', 'Ini']],
        "group_min_num_list": [3, 3, 3],
    }

    gene_loss_define = OrderedDict([('Ash', ["Aof"]),
                                    ('Vpl', ["Aof"]),
                                    ('Gel', ["Aof"]),
                                    ('Sas', ["Mgu"]),
                                    ('Cau', ["Ini"]),
                                    ('Shi', ["Mes"])])

    stem_sp_list = ['Atr', 'Pni', 'Nco', 'Osa', 'Aof',
                    'Mgu', 'Ini', 'Mes', 'Ath', 'Aco', 'Xvi']
    core_ref_sp_list = ['Ath', 'Osa']

    # read OG
    OG_dict = OG_tsv_file_parse(OG_tsv_file)

    # get conserved OG
    conserved_OG_list = get_conserved_OGs(OG_dict, conserved_arguments)

    # get lost OG in target species
    gene_lost_dict = OrderedDict()
    for target_sp in gene_loss_define:
        gene_lost_dict[target_sp] = []
        for og_id in conserved_OG_list:
            lost_flag = True
            # target species should have not this gene
            if len(OG_dict[og_id][target_sp]) != 0:
                lost_flag = False
            # ref species must have this gene
            for ref_sp_id in gene_loss_define[target_sp]:
                if len(OG_dict[og_id][ref_sp_id]) == 0:
                    lost_flag = False
            if lost_flag:
                gene_lost_dict[target_sp].append(og_id)

    for target_sp in gene_lost_dict:
        print(target_sp, len(gene_lost_dict[target_sp]))

    # re-annotated genome
    reanno_jobs_dict = {}

    for target_sp in gene_lost_dict:
        ref_sp_list = gene_loss_define[target_sp] + core_ref_sp_list
        reanno_jobs_dict[target_sp] = {ref_sp_id: []
                                       for ref_sp_id in ref_sp_list}

        for og_id in gene_lost_dict[target_sp]:
            for ref_sp_id in ref_sp_list:
                reanno_jobs_dict[target_sp][ref_sp_id] += OG_dict[og_id][ref_sp_id]

    for target_sp in reanno_jobs_dict:
        for ref_sp_id in reanno_jobs_dict[target_sp]:
            print(target_sp, ref_sp_id, len(
                reanno_jobs_dict[target_sp][ref_sp_id]))

    # genblasta + genewise
    # mkdir and copy file
    reanno_dir = work_dir + "/reanno"
    mkdir(reanno_dir, True)

    run_file_dict = {}
    for target_sp in gene_loss_define:
        run_file_dict[target_sp] = {"genome": reanno_dir + "/" + target_sp + ".genome.fasta",
                                    "gff": reanno_dir + "/" + target_sp + ".gff",
                                    }

        copy_file(species_info_dict[target_sp]['genome_file'],
                  run_file_dict[target_sp]["genome"])
        copy_file(species_info_dict[target_sp]
                  ['gff_file'], run_file_dict[target_sp]["gff"])

    # prepare genblasta
    genblasta_jobs_args_list = []
    for target_sp in reanno_jobs_dict:
        target_sp_dir = reanno_dir+"/"+target_sp
        mkdir(target_sp_dir, True)
        for ref_sp in reanno_jobs_dict[target_sp]:
            print(target_sp, ref_sp)
            ref_sp_dir = target_sp_dir + "/" + ref_sp
            mkdir(ref_sp_dir, True)
            ref_sp_aa_dict = read_fasta_by_faidx(
                species_info_dict[ref_sp]['pt_file'])
            for q_seq_id in reanno_jobs_dict[target_sp][ref_sp]:
                q_seq = ref_sp_aa_dict[q_seq_id]
                q_seq_file = ref_sp_dir+"/"+q_seq_id+".faa"
                with open(q_seq_file, 'w') as f:
                    f.write(">%s\n%s" % (q_seq.seqname, q_seq.seq))
                gba_file = ref_sp_dir+"/"+q_seq_id+".gba"
                genblasta_jobs_args_list.append(
                    (q_seq_file, run_file_dict[target_sp]['genome'], gba_file, 0.5, 20))

    for target_sp in reanno_jobs_dict:
        cmd_string = "%s/xdformat -n %s" % (wu_blast_path,
                                            run_file_dict[target_sp]["genome"])
        cmd_run(cmd_string, silence=True)

    # run genblasta
    genblasta_out = multiprocess_running(
        genblasta_run, genblasta_jobs_args_list, 80, silence=False)

    # prepare genewise
    genewise_jobs_args_list = []
    for target_sp in reanno_jobs_dict:
        target_sp_dir = reanno_dir+"/"+target_sp
        for ref_sp in reanno_jobs_dict[target_sp]:
            ref_sp_dir = target_sp_dir + "/" + ref_sp
            for q_seq_id in reanno_jobs_dict[target_sp][ref_sp]:
                q_seq_file = ref_sp_dir+"/"+q_seq_id+".faa"
                gba_file = ref_sp_dir+"/"+q_seq_id+".gba"
                wise_prefix = ref_sp_dir+"/"+q_seq_id+".wise"
                genewise_jobs_args_list.append(
                    (q_seq_file, run_file_dict[target_sp]['genome'], gba_file, '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.all_OrthoFinder/gene_loss_check/reanno/tmp', 1000, wise_prefix, False, 20))

    # run genewise
    genewise_running_out = multiprocess_running(
        genblasta_to_genewise, genewise_jobs_args_list, 80, silence=False)

    # check
    genewise_jobs_args_list = []
    for target_sp in reanno_jobs_dict:
        target_sp_dir = reanno_dir+"/"+target_sp
        for ref_sp in reanno_jobs_dict[target_sp]:
            ref_sp_dir = target_sp_dir + "/" + ref_sp
            for q_seq_id in reanno_jobs_dict[target_sp][ref_sp]:
                q_seq_file = ref_sp_dir+"/"+q_seq_id+".faa"
                gba_file = ref_sp_dir+"/"+q_seq_id+".gba"
                wise_prefix = ref_sp_dir+"/"+q_seq_id+".wise"
                gff_file = wise_prefix + ".gff3"
                if not os.path.exists(gff_file):
                    genewise_jobs_args_list.append(
                        (q_seq_file, run_file_dict[target_sp]['genome'], gba_file, '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.all_OrthoFinder/gene_loss_check/reanno/tmp', 1000, wise_prefix, False, 20))

    # rerun
    genewise_running_out = multiprocess_running(
        genblasta_to_genewise, genewise_jobs_args_list, 80, silence=False)

    # compare with gff file, filter out short one

    reanno_results_dict = {}
    for target_sp in reanno_jobs_dict:
        reanno_results_dict[target_sp] = {}
        # load annoed dict
        target_sp_annoed_dict = load_annotated_range(
            run_file_dict[target_sp]["gff"])
        target_sp_dir = reanno_dir+"/"+target_sp

        # load genewise output
        load_genewise_args_list = []
        for ref_sp in reanno_jobs_dict[target_sp]:
            ref_sp_dir = target_sp_dir + "/" + ref_sp
            for q_seq_id in reanno_jobs_dict[target_sp][ref_sp]:
                wise_prefix = ref_sp_dir+"/"+q_seq_id+".wise"
                gff_file = wise_prefix + ".gff3"
                load_genewise_args_list.append((gff_file,))

        tmp_out = multiprocess_running(
            load_genewise_gff, load_genewise_args_list, 80, silence=True)

        genewise_dict = {}
        for i in tmp_out:
            for gf_id in tmp_out[i]['output']:
                if gf_id in genewise_dict:
                    print("warnning: same gf id !")
                genewise_dict[gf_id] = tmp_out[i]['output'][gf_id]

        # remove genewise gf which not good coverage or has been annotated
        for gf_id in genewise_dict:
            gf = genewise_dict[gf_id]
            mRNA_qual = gf.sub_features[0].qualifiers
            coverage_ratio = abs(int(mRNA_qual['Target_Start'][0]) - int(mRNA_qual['Target_End'][0])) / int(
                mRNA_qual['Target_Length'][0])

            if coverage_ratio < 0.5:
                continue

            cds_range = [(cds.start, cds.end)
                         for cds in gf.sub_features[0].sub_features]
            cds_length = sum_interval_length(cds_range)

            overlaped_anno_range = []
            for tr in cds_range:
                if gf.chr_id in target_sp_annoed_dict:
                    overlaped_anno_range.extend(
                        list(target_sp_annoed_dict[gf.chr_id][gf.strand].find(tr)))
            overlaped_anno_range = merge_intervals(overlaped_anno_range)

            a, overlength, c = overlap_between_interval_set(
                cds_range, overlaped_anno_range)

            annoed_ratio = overlength/cds_length

            if annoed_ratio > 0.5:
                continue

            reanno_results_dict[target_sp][gf_id] = gf

    for target_sp in reanno_results_dict:
        tmp_dict = {}
        for gf_id in reanno_results_dict[target_sp]:
            gf = reanno_results_dict[target_sp][gf_id]
            gf_query = gf.sub_features[0].qualifiers['Target'][0]
            if gf_query not in tmp_dict:
                tmp_dict[gf_query] = {}
            tmp_dict[gf_query][gf_id] = gf
        reanno_results_dict[target_sp] = tmp_dict

    # diamond db
    diamond_dir = work_dir + "/diamond"
    mkdir(diamond_dir, True)

    all_sp_list = stem_sp_list + list(gene_loss_define.keys())

    # copy file
    for sp_id in all_sp_list:
        if sp_id not in run_file_dict:
            run_file_dict[sp_id] = {}
        run_file_dict[sp_id]["pt_file"] = diamond_dir + "/" + sp_id + ".faa"
        copy_file(species_info_dict[sp_id]['pt_file'],
                  run_file_dict[sp_id]["pt_file"])

    # makedb
    diamond_makedb_args_list = []
    for sp_id in all_sp_list:
        cmd_string = "diamond makedb --in %s/%s.faa --db %s/%s.dmnd" % (
            diamond_dir, sp_id, diamond_dir, sp_id)
        diamond_makedb_args_list.append((cmd_string, None, 1, True, None))

    tmp_out = multiprocess_running(
        cmd_run, diamond_makedb_args_list, 80, silence=True)

    # running
    diamond_args_list = []
    for i in all_sp_list:
        for j in all_sp_list:
            cmd_string = "diamond blastp -d %s/%s.dmnd -q %s/%s.faa -o %s/%s_vs_%s.bls --more-sensitive -p 1 --quiet -e 0.001 --compress 1" % (
                diamond_dir, j, diamond_dir, i, diamond_dir, i, j)
            diamond_args_list.append((cmd_string, None, 1, True, None))

    tmp_out = multiprocess_running(
        cmd_run, diamond_args_list, 80, silence=True)

    # reading bls results

    wvalue_dict = {}
    num = 0
    for i in all_sp_list:
        num += 1
        print(num, len(all_sp_list))
        wvalue_dict[i] = {}
        for j in all_sp_list:
            wvalue_dict[i][j] = {}
            blast_file = "%s/%s_vs_%s.bls.gz" % (diamond_dir, i, j)
            bls_file_parser = outfmt6_read_big(blast_file, gzip_flag=True)
            for query in bls_file_parser:
                q_def = query.qDef
                wvalue_dict[i][j][q_def] = {}
                for subject in query.hit:
                    s_def = subject.Hit_def
                    hsp = subject.hsp[0]
                    evalue = hsp.Hsp_evalue
                    wvalue_dict[i][j][q_def][s_def] = evalue_to_wvalue(
                        float(evalue))

    # tree check
    # prepare

    tree_dir = work_dir + "/tree"
    mkdir(tree_dir, True)

    ref_sp_aa_dict = {}
    for ref_sp in all_sp_list:
        ref_sp_aa_dict[ref_sp], b = read_fasta(
            species_info_dict[ref_sp]['pt_file'])

    for sp_id in gene_lost_dict:
        lost_og_list = gene_lost_dict[sp_id]
        tree_now_dir = tree_dir + "/" + sp_id
        mkdir(tree_now_dir, True)
        num = 0
        for og_id in lost_og_list:
            num += 1
            print(sp_id, og_id, num, len(lost_og_list))

            og_dir = tree_now_dir + "/" + og_id
            mkdir(og_dir, True)

            ref_seq_dict = {}
            wise_dict = {}
            ref_seq_sp_map = {}

            for ref_sp in stem_sp_list:
                for g_id in OG_dict[og_id][ref_sp]:
                    ref_seq_dict[g_id] = ref_sp_aa_dict[ref_sp][g_id].seq
                    if g_id in reanno_results_dict[sp_id]:
                        wise_dict[g_id] = reanno_results_dict[sp_id][g_id]
                    else:
                        wise_dict[g_id] = {}
                    ref_seq_sp_map[g_id] = ref_sp

            # get homology gene
            homology_gene_dir = {}
            homology_gene_sp_map = {}

            for ref_g_id in ref_seq_sp_map:
                for sp_id_now in stem_sp_list + [sp_id]:
                    if ref_g_id in wvalue_dict[ref_seq_sp_map[ref_g_id]][sp_id_now]:
                        for hit in wvalue_dict[ref_seq_sp_map[ref_g_id]][sp_id_now][ref_g_id]:
                            if hit not in ref_seq_sp_map:
                                wvalue = wvalue_dict[ref_seq_sp_map[ref_g_id]
                                                     ][sp_id_now][ref_g_id][hit]
                                if hit not in homology_gene_dir:
                                    homology_gene_dir[hit] = 0
                                    homology_gene_sp_map[hit] = sp_id_now
                                homology_gene_dir[hit] += wvalue

            homo_seq_dict = {}
            homo_seq_sp_map = {}
            for sp_id_now in stem_sp_list + [sp_id]:
                for i in sorted([i for i in homology_gene_dir if homology_gene_sp_map[i] == sp_id_now],
                                key=lambda x: homology_gene_dir[x], reverse=True)[0:5]:
                    homo_seq_dict[i] = ref_sp_aa_dict[sp_id_now][i].seq
                    homo_seq_sp_map[i] = sp_id_now

            # merge the same loci, get a new seq file
            good_genewise_hit_dir = {}
            wise_seq_dir = {}
            for q_id in wise_dict:
                for s_id in wise_dict[q_id]:
                    gf = wise_dict[q_id][s_id]
                    if gf.chr_id not in good_genewise_hit_dir:
                        good_genewise_hit_dir[gf.chr_id] = {"+": {}, "-": {}}
                    good_genewise_hit_dir[gf.chr_id][gf.strand][s_id] = gf.chr_loci.range
                    pep_file = reanno_dir + "/" + sp_id + "/" + \
                        ref_seq_sp_map[q_id] + "/" + q_id + ".wise.pep"
                    pep_dict, b = read_fasta(pep_file)
                    wise_seq_dir[s_id] = pep_dict[s_id+".pep"].seq

            new_gene_list = []
            for contig in good_genewise_hit_dir:
                for strand in good_genewise_hit_dir[contig]:
                    good_genewise_hit_dir[contig][strand] = group_by_intervals_with_overlap_threshold(
                        good_genewise_hit_dir[contig][strand], 0.5)

                    for group_id in good_genewise_hit_dir[contig][strand]:
                        genewise_hit_list = good_genewise_hit_dir[contig][strand][group_id]['list']
                        lengthest_hit = sorted(genewise_hit_list, key=lambda x: len(wise_seq_dir[x]), reverse=True)[
                            0]
                        good_genewise_hit_dir[contig][strand][group_id]['rep_hit'] = lengthest_hit
                        new_gene_list.append(lengthest_hit)

            add_seq_dict = {i: wise_seq_dir[i] for i in new_gene_list}

            pickle_info = (ref_seq_dict, ref_seq_sp_map,
                           homo_seq_dict, homo_seq_sp_map, add_seq_dict)

            seq_info_file = og_dir + "/seq_info.pyb"

            OUT = open(seq_info_file, 'wb')
            pickle.dump(pickle_info, OUT)
            OUT.close()

    # treeing

    args_list = []
    args_info_list = []
    for sp_id in gene_lost_dict:
        tree_now_dir = tree_dir + "/" + sp_id
        lost_og_list = gene_lost_dict[sp_id]
        for og_id in lost_og_list:
            args_info_list.append((sp_id, og_id))
            og_dir = tree_now_dir + "/" + og_id
            args_list.append((og_dir, sp_id, 10, 0.5, 0.5))

    tmp_out = multiprocess_running(
        tree_check, args_list, 80, None, False, args_info_list)

    # report
    with open(work_dir + "/tree_check.out", 'w') as f:
        for sp_id, og_id in tmp_out:
            flag, note = tmp_out[(sp_id, og_id)]['output']

            if isinstance(note, list):
                note_string = ",".join(note)
            else:
                note_string = note

            f.write("%s\t%s\t%s\t%s\n" %
                    (sp_id, og_id, str(flag), note_string))

    # %%
    # map Gel and Cau gene to Ath

    OG_tsv_file = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/new_pipeline/pt_file/OrthoFinder/Results_Apr13/Orthogroups/Orthogroups.tsv"
    fasttree_dir = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/new_pipeline/fasttree"
    species_tree_file = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/new_pipeline/species.txt"

    Ang_conserved_arguments = {
        "core_ref_list": ["Ath", "Osa"],
        "sp_group_lol": [["Cca", "Mgu", "Sly", "Ini", "Aco"], ["Xvi", "Aof", "Cnu", "Acom", "Eve"]],
        "group_min_num_list": [3, 3],
    }

    Eud_conserved_arguments = {
        "core_ref_list": ["Ath"],
        "sp_group_lol": [["Cca", "Mgu", "Sly", "Ini", "Aco"]],
        "group_min_num_list": [3],
    }

    conserved_function = if_conserved2
    support_threshold = 0.7
    threads = 160

    ref_sp = 'Ath'

    # Ang
    sum_dict = {}
    for sp_now in ['Osa', 'Aof', 'Ash', 'Vpl', 'Gel']:
        sum_dict[sp_now] = get_orthopairs_fasttree_pipeline(
            OG_tsv_file, fasttree_dir, species_tree_file, ref_sp, sp_now, conserved_function, Ang_conserved_arguments, support_threshold)

    # Eud
    for sp_now in ['Mgu', 'Sly', 'Ini', 'Cau']:
        sum_dict[sp_now] = get_orthopairs_fasttree_pipeline(
            OG_tsv_file, fasttree_dir, species_tree_file, ref_sp, sp_now, conserved_function, Eud_conserved_arguments, support_threshold)

    sp_id_list = ['Osa', 'Aof', 'Ash', 'Vpl',
                  'Gel'] + ['Mgu', 'Sly', 'Ini', 'Cau']

    Ath_gene_dict = {}
    for sp_now in sp_id_list:
        for ath_id in sum_dict[sp_now]:
            gene_id, og_list, conser_flag = sum_dict[sp_now][ath_id]
            if ath_id not in Ath_gene_dict:
                Ath_gene_dict[ath_id] = {}
            Ath_gene_dict[ath_id][sp_now] = gene_id

    OG_dict = OG_tsv_file_parse(OG_tsv_file)

    Ath_OG_dict = {}
    for i in OG_dict:
        for j in OG_dict[i]['Ath']:
            Ath_OG_dict[j] = i

    Ang_conserved_dict = {}
    for i in sum_dict['Osa']:
        Ang_conserved_dict[i] = sum_dict['Osa'][i][2]

    Eud_conserved_dict = {}
    for i in sum_dict['Mgu']:
        Eud_conserved_dict[i] = sum_dict['Mgu'][i][2]

    Ath_gene_annotation_tsv = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Arabidopsis_thaliana/function/all_annotation.tsv'
    Ath_gene_annotation_dict = tsv_file_dict_parse(
        Ath_gene_annotation_tsv, key_col='clean_id')

    anno_col_list = list(Ath_gene_annotation_dict[list(
        Ath_gene_annotation_dict.keys())[0]].keys())

    columns = anno_col_list[:2] + ['OG_id', 'Ang_conserved', 'Eud_conserved'] + [
        i+"_num" for i in sp_id_list] + sp_id_list + anno_col_list[2:]

    ath_anno_excel_file = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/new_pipeline/Ath_anno.xlsx"

    ath_anno_df = pd.DataFrame(columns=columns)

    num = 0
    for ath_id in Ath_gene_annotation_dict:
        anno_info = [Ath_gene_annotation_dict[ath_id][i]
                     for i in anno_col_list]

        if ath_id in Ath_gene_dict:
            num_list = [len(Ath_gene_dict[ath_id][i]) for i in sp_id_list]
            gene_id_list = [";".join(Ath_gene_dict[ath_id][i])
                            for i in sp_id_list]
            OG_id = Ath_OG_dict[ath_id]
        else:
            num_list = [0] * len(sp_id_list)
            gene_id_list = [""] * len(sp_id_list)
            OG_id = "-"

        if ath_id in Ang_conserved_dict:
            Ang_conserved_flag = Ang_conserved_dict[ath_id]
        else:
            Ang_conserved_flag = False

        if ath_id in Eud_conserved_dict:
            Eud_conserved_flag = Eud_conserved_dict[ath_id]
        else:
            Eud_conserved_flag = False

        row_data = anno_info[:2] + [OG_id, str(Ang_conserved_flag), str(
            Eud_conserved_flag)] + num_list + gene_id_list + anno_info[2:]

        ath_anno_df.loc[num] = row_data
        num = num+1

        if num % 1000 == 0:
            print(num)

    writer = pd.ExcelWriter(ath_anno_excel_file)
    ath_anno_df.to_excel(writer)
    writer.save()

    # %%
    # %%
    # map Pae and Cau gene to Ath

    OG_tsv_file = "/lustre/home/xuyuxing/Work/Phelipanche/flower_for_zhentianyin/pt_file/OrthoFinder/Results_Apr19/Orthogroups/Orthogroups.tsv"
    fasttree_dir = "/lustre/home/xuyuxing/Work/Phelipanche/flower_for_zhentianyin/fasttree"
    species_tree_file = "/lustre/home/xuyuxing/Work/Phelipanche/flower_for_zhentianyin/species.txt"

    conserved_arguments = {
        "sp_group_lol": [["Ath"], ["Cca", "Sly", "Oeu", "Sin", "Mgu"]],
        "group_min_num_list": [1, 3],
    }

    conserved_function = if_OG_conserved
    support_threshold = 0.7
    threads = 80

    ref_sp = 'Ath'

    sum_dict = {}
    for sp_now in ['Mgu', 'Sas', 'Pae']:
        sum_dict[sp_now] = get_orthopairs_fasttree_pipeline(
            OG_tsv_file, fasttree_dir, species_tree_file, ref_sp, sp_now, conserved_function, conserved_arguments, support_threshold)

    sp_id_list = ['Mgu', 'Sas', 'Pae']

    Ath_gene_dict = {}
    for sp_now in sp_id_list:
        for ath_id in sum_dict[sp_now]:
            gene_id, og_list, conser_flag = sum_dict[sp_now][ath_id]
            if ath_id not in Ath_gene_dict:
                Ath_gene_dict[ath_id] = {}
            Ath_gene_dict[ath_id][sp_now] = gene_id

    OG_dict = OG_tsv_file_parse(OG_tsv_file)

    Ath_OG_dict = {}
    for i in OG_dict:
        for j in OG_dict[i]['Ath']:
            Ath_OG_dict[j] = i

    conserved_dict = {}
    for i in sum_dict['Mgu']:
        conserved_dict[i] = sum_dict['Mgu'][i][2]

    Ath_gene_annotation_tsv = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Arabidopsis_thaliana/function/all_annotation.tsv'
    Ath_gene_annotation_dict = tsv_file_dict_parse(
        Ath_gene_annotation_tsv, key_col='clean_id')

    anno_col_list = list(Ath_gene_annotation_dict[list(
        Ath_gene_annotation_dict.keys())[0]].keys())

    columns = anno_col_list[:2] + ['OG_id', 'conserved'] + [
        i+"_num" for i in sp_id_list] + sp_id_list + anno_col_list[2:]

    ath_anno_excel_file = "/lustre/home/xuyuxing/Work/Phelipanche/flower_for_zhentianyin/Ath_anno.xlsx"

    ath_anno_df = pd.DataFrame(columns=columns)

    num = 0
    for ath_id in Ath_gene_annotation_dict:
        anno_info = [Ath_gene_annotation_dict[ath_id][i]
                     for i in anno_col_list]

        if ath_id in Ath_gene_dict:
            num_list = [len(Ath_gene_dict[ath_id][i]) for i in sp_id_list]
            gene_id_list = [";".join(Ath_gene_dict[ath_id][i])
                            for i in sp_id_list]
            OG_id = Ath_OG_dict[ath_id]
        else:
            num_list = [0] * len(sp_id_list)
            gene_id_list = [""] * len(sp_id_list)
            OG_id = "-"

        if ath_id in conserved_dict:
            conserved_flag = conserved_dict[ath_id]
        else:
            conserved_flag = False

        row_data = anno_info[:2] + [OG_id,
                                    str(conserved_flag)] + num_list + gene_id_list + anno_info[2:]

        ath_anno_df.loc[num] = row_data
        num = num+1

        if num % 1000 == 0:
            print(num)

    writer = pd.ExcelWriter(ath_anno_excel_file)
    ath_anno_df.to_excel(writer)
    writer.save()


    # %%
    # map Pae and Cau gene to Ath

    OG_tsv_file = "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/pt_file/OrthoFinder/Results_May08/Orthogroups/Orthogroups.tsv"
    fasttree_dir = "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/fasttree"
    species_tree_file = "/lustre/home/xuyuxing/Work/Other/Canrong/orthofinder/species.txt"

    conserved_arguments = {
        "sp_group_lol": [["Osa","Zma","Acom","Xvi"],["Aco","Ath","Mdo","Sly"]],
        "group_min_num_list": [2, 2],
    }

    conserved_function = if_OG_conserved
    support_threshold = 0.7
    threads = 80

    
    
    get_orthogroups_from_tree(tree_prefix, gene_tree, species_tree, gene_to_species_map_dict, conserved_function, conserved_arguments, support_threshold)




    ref_sp = 'Ath'

    sum_dict = {}
    for sp_now in ['Mgu', 'Sas', 'Pae']:
        sum_dict[sp_now] = get_orthopairs_fasttree_pipeline(
            OG_tsv_file, fasttree_dir, species_tree_file, ref_sp, sp_now, conserved_function, conserved_arguments, support_threshold)

    sp_id_list = ['Mgu', 'Sas', 'Pae']

    Ath_gene_dict = {}
    for sp_now in sp_id_list:
        for ath_id in sum_dict[sp_now]:
            gene_id, og_list, conser_flag = sum_dict[sp_now][ath_id]
            if ath_id not in Ath_gene_dict:
                Ath_gene_dict[ath_id] = {}
            Ath_gene_dict[ath_id][sp_now] = gene_id

    OG_dict = OG_tsv_file_parse(OG_tsv_file)

    Ath_OG_dict = {}
    for i in OG_dict:
        for j in OG_dict[i]['Ath']:
            Ath_OG_dict[j] = i

    conserved_dict = {}
    for i in sum_dict['Mgu']:
        conserved_dict[i] = sum_dict['Mgu'][i][2]

    Ath_gene_annotation_tsv = '/lustre/home/xuyuxing/Database/Plant_genome/clean_data/Arabidopsis_thaliana/function/all_annotation.tsv'
    Ath_gene_annotation_dict = tsv_file_dict_parse(
        Ath_gene_annotation_tsv, key_col='clean_id')

    anno_col_list = list(Ath_gene_annotation_dict[list(
        Ath_gene_annotation_dict.keys())[0]].keys())

    columns = anno_col_list[:2] + ['OG_id', 'conserved'] + [
        i+"_num" for i in sp_id_list] + sp_id_list + anno_col_list[2:]

    ath_anno_excel_file = "/lustre/home/xuyuxing/Work/Phelipanche/flower_for_zhentianyin/Ath_anno.xlsx"

    ath_anno_df = pd.DataFrame(columns=columns)

    num = 0
    for ath_id in Ath_gene_annotation_dict:
        anno_info = [Ath_gene_annotation_dict[ath_id][i]
                     for i in anno_col_list]

        if ath_id in Ath_gene_dict:
            num_list = [len(Ath_gene_dict[ath_id][i]) for i in sp_id_list]
            gene_id_list = [";".join(Ath_gene_dict[ath_id][i])
                            for i in sp_id_list]
            OG_id = Ath_OG_dict[ath_id]
        else:
            num_list = [0] * len(sp_id_list)
            gene_id_list = [""] * len(sp_id_list)
            OG_id = "-"

        if ath_id in conserved_dict:
            conserved_flag = conserved_dict[ath_id]
        else:
            conserved_flag = False

        row_data = anno_info[:2] + [OG_id,
                                    str(conserved_flag)] + num_list + gene_id_list + anno_info[2:]

        ath_anno_df.loc[num] = row_data
        num = num+1

        if num % 1000 == 0:
            print(num)

    writer = pd.ExcelWriter(ath_anno_excel_file)
    ath_anno_df.to_excel(writer)
    writer.save()


# %%

    # orthogroups level gene loss test

    OG_tsv_file = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.all_OrthoFinder/pt_file/OrthoFinder/Results_Mar16/Orthogroups/Orthogroups.tsv"
    work_dir = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.all_OrthoFinder/gene_loss_check"
    species_info_xlsx = "/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.all_OrthoFinder/gene_loss_check/species_info.xlsx"

    species_info_dict = excel_file_parse(species_info_xlsx, key_col='sp_id')

    conserved_arguments = {
        "sp_group_lol": [['Csi', 'Cnu', 'Acom', 'Bdi', 'Osa', 'Oth', 'Zma', 'Eve', 'Mac'], ['Vvi', 'Ptr', 'Mes', 'Gma', 'Fve', 'Csa', 'Ath', 'Gra'], ['Cca', 'Mgu', 'Oeu', 'Sin', 'Can', 'Sly', 'Stu', 'Ini']],
        "group_min_num_list": [3, 3, 3],
    }

    gene_loss_define = OrderedDict([('Ash', ["Aof"]),
                                    ('Vpl', ["Aof"]),
                                    ('Gel', ["Aof"]),
                                    ('Sas', ["Mgu"]),
                                    ('Cau', ["Ini"]),
                                    ('Shi', ["Mes"])])

    stem_sp_list = ['Atr', 'Pni', 'Nco', 'Osa', 'Aof',
                    'Mgu', 'Ini', 'Mes', 'Ath', 'Aco', 'Xvi']
    core_ref_sp_list = ['Ath', 'Osa']

    # read OG
    OG_dict = OG_tsv_file_parse(OG_tsv_file)

    # get conserved OG
    conserved_OG_list = get_conserved_OGs(OG_dict, conserved_arguments)

    # get lost OG in target species
    gene_lost_dict = OrderedDict()
    for target_sp in gene_loss_define:
        gene_lost_dict[target_sp] = []
        for og_id in conserved_OG_list:
            lost_flag = True
            # target species should have not this gene
            if len(OG_dict[og_id][target_sp]) != 0:
                lost_flag = False
            # ref species must have this gene
            for ref_sp_id in gene_loss_define[target_sp]:
                if len(OG_dict[og_id][ref_sp_id]) == 0:
                    lost_flag = False
            if lost_flag:
                gene_lost_dict[target_sp].append(og_id)

    for target_sp in gene_lost_dict:
        print(target_sp, len(gene_lost_dict[target_sp]))

    # re-annotated genome
    reanno_jobs_dict = {}

    for target_sp in gene_lost_dict:
        ref_sp_list = gene_loss_define[target_sp] + core_ref_sp_list
        reanno_jobs_dict[target_sp] = {ref_sp_id: []
                                       for ref_sp_id in ref_sp_list}

        for og_id in gene_lost_dict[target_sp]:
            for ref_sp_id in ref_sp_list:
                reanno_jobs_dict[target_sp][ref_sp_id] += OG_dict[og_id][ref_sp_id]

    for target_sp in reanno_jobs_dict:
        for ref_sp_id in reanno_jobs_dict[target_sp]:
            print(target_sp, ref_sp_id, len(
                reanno_jobs_dict[target_sp][ref_sp_id]))

    # genblasta + genewise
    # mkdir and copy file
    reanno_dir = work_dir + "/reanno"
    mkdir(reanno_dir, True)

    run_file_dict = {}
    for target_sp in gene_loss_define:
        run_file_dict[target_sp] = {"genome": reanno_dir + "/" + target_sp + ".genome.fasta",
                                    "gff": reanno_dir + "/" + target_sp + ".gff",
                                    }

        copy_file(species_info_dict[target_sp]['genome_file'],
                  run_file_dict[target_sp]["genome"])
        copy_file(species_info_dict[target_sp]
                  ['gff_file'], run_file_dict[target_sp]["gff"])

    # prepare genblasta
    genblasta_jobs_args_list = []
    for target_sp in reanno_jobs_dict:
        target_sp_dir = reanno_dir+"/"+target_sp
        mkdir(target_sp_dir, True)
        for ref_sp in reanno_jobs_dict[target_sp]:
            print(target_sp, ref_sp)
            ref_sp_dir = target_sp_dir + "/" + ref_sp
            mkdir(ref_sp_dir, True)
            ref_sp_aa_dict = read_fasta_by_faidx(
                species_info_dict[ref_sp]['pt_file'])
            for q_seq_id in reanno_jobs_dict[target_sp][ref_sp]:
                q_seq = ref_sp_aa_dict[q_seq_id]
                q_seq_file = ref_sp_dir+"/"+q_seq_id+".faa"
                with open(q_seq_file, 'w') as f:
                    f.write(">%s\n%s" % (q_seq.seqname, q_seq.seq))
                gba_file = ref_sp_dir+"/"+q_seq_id+".gba"
                genblasta_jobs_args_list.append(
                    (q_seq_file, run_file_dict[target_sp]['genome'], gba_file, 0.5, 20))

    for target_sp in reanno_jobs_dict:
        cmd_string = "%s/xdformat -n %s" % (wu_blast_path,
                                            run_file_dict[target_sp]["genome"])
        cmd_run(cmd_string, silence=True)

    # run genblasta
    genblasta_out = multiprocess_running(
        genblasta_run, genblasta_jobs_args_list, 80, silence=False)

    # prepare genewise
    genewise_jobs_args_list = []
    for target_sp in reanno_jobs_dict:
        target_sp_dir = reanno_dir+"/"+target_sp
        for ref_sp in reanno_jobs_dict[target_sp]:
            ref_sp_dir = target_sp_dir + "/" + ref_sp
            for q_seq_id in reanno_jobs_dict[target_sp][ref_sp]:
                q_seq_file = ref_sp_dir+"/"+q_seq_id+".faa"
                gba_file = ref_sp_dir+"/"+q_seq_id+".gba"
                wise_prefix = ref_sp_dir+"/"+q_seq_id+".wise"
                genewise_jobs_args_list.append(
                    (q_seq_file, run_file_dict[target_sp]['genome'], gba_file, '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.all_OrthoFinder/gene_loss_check/reanno/tmp', 1000, wise_prefix, False, 20))

    # run genewise
    genewise_running_out = multiprocess_running(
        genblasta_to_genewise, genewise_jobs_args_list, 80, silence=False)

    # check
    genewise_jobs_args_list = []
    for target_sp in reanno_jobs_dict:
        target_sp_dir = reanno_dir+"/"+target_sp
        for ref_sp in reanno_jobs_dict[target_sp]:
            ref_sp_dir = target_sp_dir + "/" + ref_sp
            for q_seq_id in reanno_jobs_dict[target_sp][ref_sp]:
                q_seq_file = ref_sp_dir+"/"+q_seq_id+".faa"
                gba_file = ref_sp_dir+"/"+q_seq_id+".gba"
                wise_prefix = ref_sp_dir+"/"+q_seq_id+".wise"
                gff_file = wise_prefix + ".gff3"
                if not os.path.exists(gff_file):
                    genewise_jobs_args_list.append(
                        (q_seq_file, run_file_dict[target_sp]['genome'], gba_file, '/lustre/home/xuyuxing/Work/Gel/Gene_Loss2/S44_version3/OrthoFinder/3.all_OrthoFinder/gene_loss_check/reanno/tmp', 1000, wise_prefix, False, 20))

    # rerun
    genewise_running_out = multiprocess_running(
        genblasta_to_genewise, genewise_jobs_args_list, 80, silence=False)

    # compare with gff file, filter out short one

    reanno_results_dict = {}
    for target_sp in reanno_jobs_dict:
        reanno_results_dict[target_sp] = {}
        # load annoed dict
        target_sp_annoed_dict = load_annotated_range(
            run_file_dict[target_sp]["gff"])
        target_sp_dir = reanno_dir+"/"+target_sp

        # load genewise output
        load_genewise_args_list = []
        for ref_sp in reanno_jobs_dict[target_sp]:
            ref_sp_dir = target_sp_dir + "/" + ref_sp
            for q_seq_id in reanno_jobs_dict[target_sp][ref_sp]:
                wise_prefix = ref_sp_dir+"/"+q_seq_id+".wise"
                gff_file = wise_prefix + ".gff3"
                load_genewise_args_list.append((gff_file,))

        tmp_out = multiprocess_running(
            load_genewise_gff, load_genewise_args_list, 80, silence=True)

        genewise_dict = {}
        for i in tmp_out:
            for gf_id in tmp_out[i]['output']:
                if gf_id in genewise_dict:
                    print("warnning: same gf id !")
                genewise_dict[gf_id] = tmp_out[i]['output'][gf_id]

        # remove genewise gf which not good coverage or has been annotated
        for gf_id in genewise_dict:
            gf = genewise_dict[gf_id]
            mRNA_qual = gf.sub_features[0].qualifiers
            coverage_ratio = abs(int(mRNA_qual['Target_Start'][0]) - int(mRNA_qual['Target_End'][0])) / int(
                mRNA_qual['Target_Length'][0])

            if coverage_ratio < 0.5:
                continue

            cds_range = [(cds.start, cds.end)
                         for cds in gf.sub_features[0].sub_features]
            cds_length = sum_interval_length(cds_range)

            overlaped_anno_range = []
            for tr in cds_range:
                if gf.chr_id in target_sp_annoed_dict:
                    overlaped_anno_range.extend(
                        list(target_sp_annoed_dict[gf.chr_id][gf.strand].find(tr)))
            overlaped_anno_range = merge_intervals(overlaped_anno_range)

            a, overlength, c = overlap_between_interval_set(
                cds_range, overlaped_anno_range)

            annoed_ratio = overlength/cds_length

            if annoed_ratio > 0.5:
                continue

            reanno_results_dict[target_sp][gf_id] = gf

    for target_sp in reanno_results_dict:
        tmp_dict = {}
        for gf_id in reanno_results_dict[target_sp]:
            gf = reanno_results_dict[target_sp][gf_id]
            gf_query = gf.sub_features[0].qualifiers['Target'][0]
            if gf_query not in tmp_dict:
                tmp_dict[gf_query] = {}
            tmp_dict[gf_query][gf_id] = gf
        reanno_results_dict[target_sp] = tmp_dict

    # diamond db
    diamond_dir = work_dir + "/diamond"
    mkdir(diamond_dir, True)

    all_sp_list = stem_sp_list + list(gene_loss_define.keys())

    # copy file
    for sp_id in all_sp_list:
        if sp_id not in run_file_dict:
            run_file_dict[sp_id] = {}
        run_file_dict[sp_id]["pt_file"] = diamond_dir + "/" + sp_id + ".faa"
        copy_file(species_info_dict[sp_id]['pt_file'],
                  run_file_dict[sp_id]["pt_file"])

    # makedb
    diamond_makedb_args_list = []
    for sp_id in all_sp_list:
        cmd_string = "diamond makedb --in %s/%s.faa --db %s/%s.dmnd" % (
            diamond_dir, sp_id, diamond_dir, sp_id)
        diamond_makedb_args_list.append((cmd_string, None, 1, True, None))

    tmp_out = multiprocess_running(
        cmd_run, diamond_makedb_args_list, 80, silence=True)

    # running
    diamond_args_list = []
    for i in all_sp_list:
        for j in all_sp_list:
            cmd_string = "diamond blastp -d %s/%s.dmnd -q %s/%s.faa -o %s/%s_vs_%s.bls --more-sensitive -p 1 --quiet -e 0.001 --compress 1" % (
                diamond_dir, j, diamond_dir, i, diamond_dir, i, j)
            diamond_args_list.append((cmd_string, None, 1, True, None))

    tmp_out = multiprocess_running(
        cmd_run, diamond_args_list, 80, silence=True)

    # reading bls results

    wvalue_dict = {}
    num = 0
    for i in all_sp_list:
        num += 1
        print(num, len(all_sp_list))
        wvalue_dict[i] = {}
        for j in all_sp_list:
            wvalue_dict[i][j] = {}
            blast_file = "%s/%s_vs_%s.bls.gz" % (diamond_dir, i, j)
            bls_file_parser = outfmt6_read_big(blast_file, gzip_flag=True)
            for query in bls_file_parser:
                q_def = query.qDef
                wvalue_dict[i][j][q_def] = {}
                for subject in query.hit:
                    s_def = subject.Hit_def
                    hsp = subject.hsp[0]
                    evalue = hsp.Hsp_evalue
                    wvalue_dict[i][j][q_def][s_def] = evalue_to_wvalue(
                        float(evalue))

    # tree check
    # prepare

    tree_dir = work_dir + "/tree"
    mkdir(tree_dir, True)

    ref_sp_aa_dict = {}
    for ref_sp in all_sp_list:
        ref_sp_aa_dict[ref_sp], b = read_fasta(
            species_info_dict[ref_sp]['pt_file'])

    for sp_id in gene_lost_dict:
        lost_og_list = gene_lost_dict[sp_id]
        tree_now_dir = tree_dir + "/" + sp_id
        mkdir(tree_now_dir, True)
        num = 0
        for og_id in lost_og_list:
            num += 1
            print(sp_id, og_id, num, len(lost_og_list))

            og_dir = tree_now_dir + "/" + og_id
            mkdir(og_dir, True)

            ref_seq_dict = {}
            wise_dict = {}
            ref_seq_sp_map = {}

            for ref_sp in stem_sp_list:
                for g_id in OG_dict[og_id][ref_sp]:
                    ref_seq_dict[g_id] = ref_sp_aa_dict[ref_sp][g_id].seq
                    if g_id in reanno_results_dict[sp_id]:
                        wise_dict[g_id] = reanno_results_dict[sp_id][g_id]
                    else:
                        wise_dict[g_id] = {}
                    ref_seq_sp_map[g_id] = ref_sp

            # get homology gene
            homology_gene_dir = {}
            homology_gene_sp_map = {}

            for ref_g_id in ref_seq_sp_map:
                for sp_id_now in stem_sp_list + [sp_id]:
                    if ref_g_id in wvalue_dict[ref_seq_sp_map[ref_g_id]][sp_id_now]:
                        for hit in wvalue_dict[ref_seq_sp_map[ref_g_id]][sp_id_now][ref_g_id]:
                            if hit not in ref_seq_sp_map:
                                wvalue = wvalue_dict[ref_seq_sp_map[ref_g_id]
                                                     ][sp_id_now][ref_g_id][hit]
                                if hit not in homology_gene_dir:
                                    homology_gene_dir[hit] = 0
                                    homology_gene_sp_map[hit] = sp_id_now
                                homology_gene_dir[hit] += wvalue

            homo_seq_dict = {}
            homo_seq_sp_map = {}
            for sp_id_now in stem_sp_list + [sp_id]:
                for i in sorted([i for i in homology_gene_dir if homology_gene_sp_map[i] == sp_id_now],
                                key=lambda x: homology_gene_dir[x], reverse=True)[0:5]:
                    homo_seq_dict[i] = ref_sp_aa_dict[sp_id_now][i].seq
                    homo_seq_sp_map[i] = sp_id_now

            # merge the same loci, get a new seq file
            good_genewise_hit_dir = {}
            wise_seq_dir = {}
            for q_id in wise_dict:
                for s_id in wise_dict[q_id]:
                    gf = wise_dict[q_id][s_id]
                    if gf.chr_id not in good_genewise_hit_dir:
                        good_genewise_hit_dir[gf.chr_id] = {"+": {}, "-": {}}
                    good_genewise_hit_dir[gf.chr_id][gf.strand][s_id] = gf.chr_loci.range
                    pep_file = reanno_dir + "/" + sp_id + "/" + \
                        ref_seq_sp_map[q_id] + "/" + q_id + ".wise.pep"
                    pep_dict, b = read_fasta(pep_file)
                    wise_seq_dir[s_id] = pep_dict[s_id+".pep"].seq

            new_gene_list = []
            for contig in good_genewise_hit_dir:
                for strand in good_genewise_hit_dir[contig]:
                    good_genewise_hit_dir[contig][strand] = group_by_intervals_with_overlap_threshold(
                        good_genewise_hit_dir[contig][strand], 0.5)

                    for group_id in good_genewise_hit_dir[contig][strand]:
                        genewise_hit_list = good_genewise_hit_dir[contig][strand][group_id]['list']
                        lengthest_hit = sorted(genewise_hit_list, key=lambda x: len(wise_seq_dir[x]), reverse=True)[
                            0]
                        good_genewise_hit_dir[contig][strand][group_id]['rep_hit'] = lengthest_hit
                        new_gene_list.append(lengthest_hit)

            add_seq_dict = {i: wise_seq_dir[i] for i in new_gene_list}

            pickle_info = (ref_seq_dict, ref_seq_sp_map,
                           homo_seq_dict, homo_seq_sp_map, add_seq_dict)

            seq_info_file = og_dir + "/seq_info.pyb"

            OUT = open(seq_info_file, 'wb')
            pickle.dump(pickle_info, OUT)
            OUT.close()

    # treeing

    args_list = []
    args_info_list = []
    for sp_id in gene_lost_dict:
        tree_now_dir = tree_dir + "/" + sp_id
        lost_og_list = gene_lost_dict[sp_id]
        for og_id in lost_og_list:
            args_info_list.append((sp_id, og_id))
            og_dir = tree_now_dir + "/" + og_id
            args_list.append((og_dir, sp_id, 10, 0.5, 0.5))

    tmp_out = multiprocess_running(
        tree_check, args_list, 80, None, False, args_info_list)

    # report
    with open(work_dir + "/tree_check.out", 'w') as f:
        for sp_id, og_id in tmp_out:
            flag, note = tmp_out[(sp_id, og_id)]['output']

            if isinstance(note, list):
                note_string = ",".join(note)
            else:
                note_string = note

            f.write("%s\t%s\t%s\t%s\n" %
                    (sp_id, og_id, str(flag), note_string))



import matplotlib

matplotlib.use('Agg')

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
from matplotlib import cm
from lib.common.fileIO import tsv_file_dict_parse
import matplotlib.gridspec as gridspec


def add_high(data, num, strand):
    new_array = []
    for i in range(0, int(data.shape[0])):
        for j in range(0, num):
            if strand == "+":
                new_array.append(data[i,])
            else:
                new_array.append(list(reversed(data[i,])))
    return np.array(new_array)


file = '/lustre/home/xuyuxing/Work/Other/Pseudouracil/all_tran/printdata2.txt'

data_dir = tsv_file_dict_parse(file)

intersection_list = {"WT": ("S02_S01", "S06_S05", "S10_S26"),
                     "sT": ("S04_S03", "S08_S07", "S12_S11"),
                     "Wm": ("S18_S17", "S22_S30", "S32_S31"),
                     "sm": ("S14_S13", "S16_S15", "S20_S19")}

id = 'Chr3_5.5S'
strand = "+"

peak_data_WT = []
for peak_tab in intersection_list['WT']:
    peak_data_WT.append([float(data_dir[i][peak_tab]) for i in data_dir if data_dir[i]['Gene_id'] == id])

peak_data_sT = []
for peak_tab in intersection_list['sT']:
    peak_data_sT.append([float(data_dir[i][peak_tab]) for i in data_dir if data_dir[i]['Gene_id'] == id])

peak_data_WT = add_high(np.array(peak_data_WT), 3, strand)
peak_data_sT = add_high(np.array(peak_data_sT), 3, strand)

# plt.style.use('ggplot')

# Make plot with vertical (default) colorbar
# fig, (ax0, ax1) = plt.subplots(2, 1)

# plt.figure(figsize=(int(len(peak_data_WT[0]) / 100), 10))

# gs = gridspec.GridSpec(17, 1, height_ratios=[200, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
gs = gridspec.GridSpec(13, 1, height_ratios=[10, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])

ax0 = plt.subplot(gs[0])
ax1 = plt.subplot(gs[1], sharex=ax0)
ax2 = plt.subplot(gs[2], sharex=ax0)
ax3 = plt.subplot(gs[3], sharex=ax0)
ax4 = plt.subplot(gs[4], sharex=ax0)
ax5 = plt.subplot(gs[5], sharex=ax0)
ax6 = plt.subplot(gs[6], sharex=ax0)
ax7 = plt.subplot(gs[7], sharex=ax0)
ax8 = plt.subplot(gs[8], sharex=ax0)
ax9 = plt.subplot(gs[9], sharex=ax0)
ax10 = plt.subplot(gs[10], sharex=ax0)
ax11 = plt.subplot(gs[11], sharex=ax0)
ax12 = plt.subplot(gs[12], sharex=ax0)

# plt.set_size_inches(18.5, 10.5)

# plt.subplots_adjust(wspace=0, hspace=-100)

cax0 = ax0.imshow(np.array(list(peak_data_WT) + list(peak_data_sT)), interpolation='nearest', cmap=cm.YlOrRd)
# plt.colorbar(cax0, ax=ax0)
# cax0 = ax0.imshow(peak_data_WT, interpolation='nearest', cmap=cm.YlOrRd)
# cax0 = ax0.imshow(peak_data_WT)
ax0.spines['top'].set_visible(False)
ax0.spines['right'].set_visible(False)
ax0.spines['bottom'].set_visible(False)
ax0.spines['left'].set_visible(False)
ax0.get_xaxis().set_visible(False)
ax0.set_xlim(0, len(peak_data_WT[0]))
line2 = ax0.axhline(y=8.5, color='w', linewidth=2)

WT_coverage_p = []
WT_coverage_n = []
for peak_tab in intersection_list['WT']:
    CMC_p, CMC_n = peak_tab.split('_')
    WT_coverage_p.append([float(data_dir[i][CMC_p]) for i in data_dir if data_dir[i]['Gene_id'] == id])
    WT_coverage_n.append([float(data_dir[i][CMC_n]) for i in data_dir if data_dir[i]['Gene_id'] == id])

num = 0
for ax_tmp1, ax_tmp2 in [(ax1, ax2), (ax3, ax4), (ax5, ax6)]:

    ax_tmp1.set_xlim(0, len(WT_coverage_p[num]))

    if strand == "+":
        data = WT_coverage_p[num]
    else:
        data = list(reversed(WT_coverage_p[num]))

    cax1 = ax_tmp1.plot(range(0, len(WT_coverage_p[num])), data, 'b', lw=0.8)
    # cax1 = ax1.imshow(peak_data_sT)
    ax_tmp1.spines['top'].set_visible(False)
    ax_tmp1.spines['right'].set_visible(False)
    ax_tmp1.spines['bottom'].set_visible(False)
    ax_tmp1.spines['left'].set_visible(False)
    ax_tmp1.get_xaxis().set_visible(False)

    ax_tmp2.set_xlim(0, len(WT_coverage_n))

    if strand == "+":
        data = WT_coverage_n[num]
    else:
        data = list(reversed(WT_coverage_n[num]))

    cax2 = ax_tmp2.plot(range(0, len(WT_coverage_n[num])), data, 'b--', lw=0.8)
    # cax1 = ax1.imshow(peak_data_sT)
    ax_tmp2.spines['top'].set_visible(False)
    ax_tmp2.spines['right'].set_visible(False)
    ax_tmp2.spines['bottom'].set_visible(False)
    ax_tmp2.spines['left'].set_visible(False)
    ax_tmp2.get_xaxis().set_visible(False)

    num = num + 1

sT_coverage_p = []
sT_coverage_n = []
for peak_tab in intersection_list['sT']:
    CMC_p, CMC_n = peak_tab.split('_')
    sT_coverage_p.append([float(data_dir[i][CMC_p]) for i in data_dir if data_dir[i]['Gene_id'] == id])
    sT_coverage_n.append([float(data_dir[i][CMC_n]) for i in data_dir if data_dir[i]['Gene_id'] == id])

num = 0
for ax_tmp1, ax_tmp2 in [(ax7, ax8), (ax9, ax10), (ax11, ax12)]:
    ax_tmp1.set_xlim(0, len(sT_coverage_p[num]))

    if strand == "+":
        data = sT_coverage_p[num]
    else:
        data = list(reversed(sT_coverage_p[num]))

    cax1 = ax_tmp1.plot(range(0, len(sT_coverage_p[num])), data, 'g', lw=0.8)
    # cax1 = ax1.imshow(peak_data_sT)
    ax_tmp1.spines['top'].set_visible(False)
    ax_tmp1.spines['right'].set_visible(False)
    ax_tmp1.spines['bottom'].set_visible(False)
    ax_tmp1.spines['left'].set_visible(False)
    ax_tmp1.get_xaxis().set_visible(False)

    ax_tmp2.set_xlim(0, len(sT_coverage_p[num]))

    if strand == "+":
        data = sT_coverage_n[num]
    else:
        data = list(reversed(sT_coverage_n[num]))

    cax2 = ax_tmp2.plot(range(0, len(sT_coverage_n[num])), data, 'g--', lw=0.8)
    # cax1 = ax1.imshow(peak_data_sT)
    ax_tmp2.spines['top'].set_visible(False)
    ax_tmp2.spines['right'].set_visible(False)
    ax_tmp2.spines['bottom'].set_visible(False)
    ax_tmp2.spines['left'].set_visible(False)
    if not num == 2:
        ax_tmp2.get_xaxis().set_visible(False)

    num = num + 1
xmajorLocator = MultipleLocator(200)

for ax_tmp in [ax0, ax1, ax2, ax3, ax4, ax5, ax6, ax7, ax8, ax9, ax10, ax11, ax12]:
    ax_tmp.xaxis.set_major_locator(xmajorLocator)
    ax_tmp.yaxis.set_major_locator(plt.NullLocator())

# plt.axis('off')

# plt.grid(False)
plt.savefig('/lustre/home/xuyuxing/Work/Other/Pseudouracil/all_tran/' + id + '.pdf', dpi=1000)
# fig.savefig('25S rRNA.pdf', dpi=1000)

plt.show()

####

import matplotlib

matplotlib.use('Agg')

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import MultipleLocator, FormatStrFormatter
from matplotlib import cm
from lib.common.fileIO import tsv_file_dict_parse
import matplotlib.gridspec as gridspec


def add_high(data, num, strand):
    new_array = []
    for i in range(0, int(data.shape[0])):
        for j in range(0, num):
            if strand == "+":
                new_array.append(data[i,])
            else:
                new_array.append(list(reversed(data[i,])))
    return np.array(new_array)


file = '/lustre/home/xuyuxing/Work/Other/Pseudouracil/all_tran/printer3.txt'

data_dir = tsv_file_dict_parse(file)

intersection_list = {"WT": ("S02_S01", "S06_S05", "S10_S26"),
                     "sT": ("S04_S03", "S08_S07", "S12_S11"),
                     "Wm": ("S18_S17", "S22_S30", "S32_S31"),
                     "sm": ("S14_S13", "S16_S15", "S20_S19")}

id = 'Chr2_5.5S'
strand = "+"

peak_data_WT = []
for peak_tab in intersection_list['WT']:
    peak_data_WT.append([float(data_dir[i][peak_tab]) for i in data_dir if data_dir[i]['Gene_id'] == id])

peak_data_sT = []
for peak_tab in intersection_list['sT']:
    peak_data_sT.append([float(data_dir[i][peak_tab]) for i in data_dir if data_dir[i]['Gene_id'] == id])

peak_data_WT = add_high(np.array(peak_data_WT), 10, strand)
peak_data_sT = add_high(np.array(peak_data_sT), 10, strand)

#peak_data_WT = add_high(np.array(peak_data_WT), 40, strand)
#peak_data_sT = add_high(np.array(peak_data_sT), 40, strand)


# plt.style.use('ggplot')

# Make plot with vertical (default) colorbar
# fig, (ax0, ax1) = plt.subplots(2, 1)

#plt.figure(figsize=(25, 10))

# gs = gridspec.GridSpec(17, 1, height_ratios=[200, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
gs = gridspec.GridSpec(13, 1, height_ratios=[5, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])
#gs = gridspec.GridSpec(13, 1, height_ratios=[25, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1])

ax0 = plt.subplot(gs[0])
ax1 = plt.subplot(gs[1], sharex=ax0)
ax2 = plt.subplot(gs[2], sharex=ax0)
ax3 = plt.subplot(gs[3], sharex=ax0)
ax4 = plt.subplot(gs[4], sharex=ax0)
ax5 = plt.subplot(gs[5], sharex=ax0)
ax6 = plt.subplot(gs[6], sharex=ax0)
ax7 = plt.subplot(gs[7], sharex=ax0)
ax8 = plt.subplot(gs[8], sharex=ax0)
ax9 = plt.subplot(gs[9], sharex=ax0)
ax10 = plt.subplot(gs[10], sharex=ax0)
ax11 = plt.subplot(gs[11], sharex=ax0)
ax12 = plt.subplot(gs[12], sharex=ax0)
# plt.set_size_inches(18.5, 10.5)

# plt.subplots_adjust(wspace=0, hspace=-100)

cax0 = ax0.imshow(np.array(list(peak_data_WT) + list(peak_data_sT)), interpolation='nearest', cmap=cm.YlOrRd)
plt.colorbar(cax0, ax=ax0)
# cax0 = ax0.imshow(peak_data_WT, interpolation='nearest', cmap=cm.YlOrRd)
# cax0 = ax0.imshow(peak_data_WT)
ax0.spines['top'].set_visible(False)
ax0.spines['right'].set_visible(False)
ax0.spines['bottom'].set_visible(False)
ax0.spines['left'].set_visible(False)
ax0.get_xaxis().set_visible(False)
ax0.set_xlim(0, len(peak_data_WT[0]))
line2 = ax0.axhline(y=29, color='w', linewidth=2)

WT_coverage_p = []
WT_coverage_n = []
for peak_tab in intersection_list['WT']:
    CMC_p, CMC_n = peak_tab.split('_')
    WT_coverage_p.append([float(data_dir[i][CMC_p]) for i in data_dir if data_dir[i]['Gene_id'] == id])
    WT_coverage_n.append([float(data_dir[i][CMC_n]) for i in data_dir if data_dir[i]['Gene_id'] == id])

num = 0
for ax_tmp1, ax_tmp2 in [(ax1, ax2), (ax3, ax4), (ax5, ax6)]:

    ax_tmp1.set_xlim(0, len(WT_coverage_p[num]))

    if strand == "+":
        data = WT_coverage_p[num]
    else:
        data = list(reversed(WT_coverage_p[num]))

    cax1 = ax_tmp1.plot(range(0, len(WT_coverage_p[num])), data, 'b', lw=0.4)
    # cax1 = ax1.imshow(peak_data_sT)
    ax_tmp1.spines['top'].set_visible(False)
    ax_tmp1.spines['right'].set_visible(False)
    ax_tmp1.spines['bottom'].set_visible(False)
    ax_tmp1.spines['left'].set_visible(False)
    ax_tmp1.get_xaxis().set_visible(False)

    ax_tmp2.set_xlim(0, len(WT_coverage_n))

    if strand == "+":
        data = WT_coverage_n[num]
    else:
        data = list(reversed(WT_coverage_n[num]))

    cax2 = ax_tmp2.plot(range(0, len(WT_coverage_n[num])), data, 'b--', lw=0.4)
    # cax1 = ax1.imshow(peak_data_sT)
    ax_tmp2.spines['top'].set_visible(False)
    ax_tmp2.spines['right'].set_visible(False)
    ax_tmp2.spines['bottom'].set_visible(False)
    ax_tmp2.spines['left'].set_visible(False)
    ax_tmp2.get_xaxis().set_visible(False)

    num = num + 1

sT_coverage_p = []
sT_coverage_n = []
for peak_tab in intersection_list['sT']:
    CMC_p, CMC_n = peak_tab.split('_')
    sT_coverage_p.append([float(data_dir[i][CMC_p]) for i in data_dir if data_dir[i]['Gene_id'] == id])
    sT_coverage_n.append([float(data_dir[i][CMC_n]) for i in data_dir if data_dir[i]['Gene_id'] == id])

num = 0
for ax_tmp1, ax_tmp2 in [(ax7, ax8),(ax9, ax10), (ax11, ax12)]:
    ax_tmp1.set_xlim(0, len(sT_coverage_p[num]))

    if strand == "+":
        data = sT_coverage_p[num]
    else:
        data = list(reversed(sT_coverage_p[num]))

    cax1 = ax_tmp1.plot(range(0, len(sT_coverage_p[num])), data, 'g', lw=0.4)
    # cax1 = ax1.imshow(peak_data_sT)
    ax_tmp1.spines['top'].set_visible(False)
    ax_tmp1.spines['right'].set_visible(False)
    ax_tmp1.spines['bottom'].set_visible(False)
    ax_tmp1.spines['left'].set_visible(False)
    ax_tmp1.get_xaxis().set_visible(False)

    ax_tmp2.set_xlim(0, len(sT_coverage_p[num]))

    if strand == "+":
        data = sT_coverage_n[num]
    else:
        data = list(reversed(sT_coverage_n[num]))

    cax2 = ax_tmp2.plot(range(0, len(sT_coverage_n[num])), data, 'g--', lw=0.4)
    # cax1 = ax1.imshow(peak_data_sT)
    ax_tmp2.spines['top'].set_visible(False)
    ax_tmp2.spines['right'].set_visible(False)
    ax_tmp2.spines['bottom'].set_visible(False)
    ax_tmp2.spines['left'].set_visible(False)
    if not num == 2:
        ax_tmp2.get_xaxis().set_visible(False)

    num = num + 1
xmajorLocator = MultipleLocator(200)

for ax_tmp in [ax0, ax1, ax2, ax3, ax4, ax5, ax6, ax7, ax8, ax9, ax10, ax11, ax12]:
    ax_tmp.xaxis.set_major_locator(xmajorLocator)
    ax_tmp.yaxis.set_major_locator(plt.NullLocator())

# plt.axis('off')

# plt.grid(False)
plt.savefig('/lustre/home/xuyuxing/Work/Other/Pseudouracil/all_tran/' + id + '.bar.pdf', dpi=1000)
# fig.savefig('25S rRNA.pdf', dpi=1000)

plt.show()



if __name__ == '__main__':
    import argparse

    # argument parse
    # ---------------------------------------------------------
    parser = argparse.ArgumentParser(
        prog='GenomeTools',
    )

    subparsers = parser.add_subparsers(
        title='subcommands', dest="subcommand_name")

    # argparse for GenomeFormater
    parser_a = subparsers.add_parser('GenomeFormater',
                                     help='make genome info to a strandard fromat\n',
                                     description='')

    parser_a.add_argument('gff_file', type=str,
                          help='Path of genome feature gff file')
    parser_a.add_argument('rename_prefix', type=str,
                          help='prefix of rename, like: Ath')
    parser_a.add_argument('output_dir', type=str, help='Path of output dir')
    parser_a.add_argument('-g', '--genome_fasta_file', type=str,
                          help='Path of genome fasta file', default=None)
    parser_a.add_argument('-p', '--protein_fasta_file', type=str,
                          help='Path of protein fasta file', default=None)
    parser_a.add_argument('-c', '--cds_fasta_file', type=str,
                          help='Path of CDS fasta file', default=None)
    parser_a.add_argument("-m", "--mode", help='raw data mode, which related to source of data',
                          default="normal", choices=['normal', 'phytozome', 'ncbi'])
    parser_a.add_argument("-k", "--keep_raw_contig_id", help="keep raw contig id, just change gene name",
                          action='store_true')

    # argparse for FragmentGenome
    parser_a = subparsers.add_parser('FragmentGenome',
                                     help='cut genome to fragment')
    parser_a.add_argument(
        "genome_file", help="Path of genome file with fasta format", type=str)
    parser_a.add_argument(
        "output_file", help="Path of cutted genome file", type=str)
    parser_a.add_argument("step", help="step of genome cutting", type=int)
    parser_a.add_argument(
        "length", help="length of sequence of cutted genome", type=int)

    parser_a.add_argument("-s", "--Consider_scaffold", help="If consider scaffold in the genome (default as True)",
                          default=True)

    # argparse for ReGetContig
    parser_a = subparsers.add_parser('ReGetContig',
                                     help='cut scaffold to contig')
    parser_a.add_argument(
        "scaff_file", help="Path of scaffold version to contig again", type=str)
    parser_a.add_argument(
        "output_gff3", help="Path of cutted contig gff file", type=str)
    parser_a.add_argument(
        "output_fasta", help="Path of cutted contig fasta file", type=str)
    parser_a.add_argument(
        "-t", "--threads", help="threads number (default as 5)", default=5, type=int)

    args = parser.parse_args()
    args_dict = vars(args)

    # ---------------------------------------------------------
    # command detail

    # FragmentGenome
    if args_dict["subcommand_name"] == "FragmentGenome":
        import lib.common.genome.seq_base as fa

        genome_file = args.genome_file
        output_file = args.output_file
        step = args.step
        length = args.length
        Consider_scaffold = args.Consider_scaffold

        with open(output_file, 'w') as f:
            for i in fa.read_fasta_big(genome_file):
                record_tmp = i
                seq_name = record_tmp.seqname_short()
                sequence = record_tmp.seq
                for start, sub_seq in fa.scaffold_cutting(sequence, step=step, length=length,
                                                          Consider_scaffold=Consider_scaffold):
                    end = len(sub_seq) + start - 1
                    sub_seq_name = seq_name + "_" + str(start) + "-" + str(end)
                    f.write(">%s\n%s\n" % (sub_seq_name, sub_seq))

    # ReGetContig
    elif args_dict["subcommand_name"] == "ReGetContig":
        from src.xuyuxing.GenomeTools.ReGetContig import ReGetContig_main
        ReGetContig_main(args)

    # GenomeFormater
    elif args_dict["subcommand_name"] == "GenomeFormater":
        from src.xuyuxing.GenomeTools.GenomeFormater import GenomeFormater_main
        GenomeFormater_main(args)





import re
from collections import OrderedDict
from lib.common.os import cmd_run
from lib.common.genome.seq_base import read_fasta_by_faidx
from lib.common.math.interval import merge_intervals, interval_minus_set
from interlap import InterLap


def hmmscan_output_reader(hmmscan_output):
    """http://eddylab.org/software/hmmer/Userguide.pdf"""
    col_name = [
        "target_name", "target_accession", "target_len", "query_name", "query_accession", 'query_len',
        "full_seq_e_value", "full_seq_score", "full_seq_bias", "domain_rank", "total_domain",
        "c_Evalue", "i_Evalue", "domain_score", "domain_bias", "hmm_from", "hmm_to", "ali_from", "ali_to",
        "env_from", "env_to", "acc", "target_description"
    ]

    output_dir = OrderedDict()
    num = 0
    with open(hmmscan_output, 'r') as f:
        for each_line in f:
            if re.match(r'^#', each_line):
                continue
            each_line = re.sub('\n', '', each_line)
            info = each_line.split()
            output_dir["ID_" + str(num)] = {}
            for i in range(0, len(col_name)):
                output_dir["ID_" + str(num)][col_name[i]] = info[i]
            num = num + 1

    return output_dir


def seq_split_by_star(sequence):
    range_list = []
    start = 0
    end = 0
    for i in range(0, len(sequence)):
        if sequence[i] == '*':
            end = i - 1
            if i == start:
                start = i + 1
                continue
            range_list.append((start, end))
            start = i + 1
            end = i + 1
        if sequence[i] != '*' and i == len(sequence) - 1:
            range_list.append((start, i))

    output_list = []
    for i in range_list:
        output_list.append((i[0] + 1, i[1] + 1))

    return output_list


def get_aa_seq(genome_seq_file, transeq_output, hmmscan_output):
    transeq_dict = read_fasta_by_faidx(transeq_output)
    hmmscan_dict = hmmscan_output_reader(hmmscan_output)

    split_star_dict = {}
    for seq_name in transeq_dict:
        split_star_dict[seq_name] = InterLap()
        split_star_dict[seq_name].update(seq_split_by_star(str(transeq_dict[seq_name].seq)))

    output_dir = {}
    for seq_name in read_fasta_by_faidx(genome_seq_file):
        seq_trans = {
            "+": [seq_name + "_1", seq_name + "_2", seq_name + "_3"],
            "-": [seq_name + "_4", seq_name + "_5", seq_name + "_6"]
        }

        seq_trans_hmm = {"+": [],
                         "-": []}

        for i in hmmscan_dict:
            q_name = hmmscan_dict[i]['query_name']
            if q_name in seq_trans['+']:
                seq_trans_hmm["+"].append((int(hmmscan_dict[i]['env_from']), int(hmmscan_dict[i]['env_to']), q_name))
            if q_name in seq_trans['-']:
                seq_trans_hmm["-"].append((int(hmmscan_dict[i]['env_from']), int(hmmscan_dict[i]['env_to']), q_name))

        output_seq = {}
        for strand in ["+", "-"]:
            used_list = []
            for start, end, info in seq_trans_hmm[strand]:
                range_tmp = (start, end)
                get_range_list = interval_minus_set(range_tmp, merge_intervals([(i, j) for i, j, k in used_list], True))
                for i in get_range_list:
                    used_list.append((i[0], i[1], info))

            expand_list = []
            for s, e, q_name in used_list:
                expand_list.append((list(split_star_dict[q_name].find((s, e))), q_name))

            # add expand
            for expand_list_now, q_name in expand_list:
                for range_tmp in expand_list_now:
                    get_range_list = interval_minus_set(range_tmp,
                                                        merge_intervals([(i, j) for i, j, k in used_list], True))
                    for i in get_range_list:
                        used_list.append((i[0], i[1], q_name))

            # print(used_list)

            seq = ""
            for i in sorted(used_list, key=lambda x: x[0]):
                seq = seq + str(transeq_dict[i[2]].seq)[i[0] - 1:i[1]]

            seq = re.sub('\*', '', seq)
            output_seq[strand] = seq

        output_dir[seq_name] = output_seq

    return output_dir

#
# # input
# genome_seq_file = '/lustre/home/xuyuxing/Work/Cuscuta_HGT/HGT_Nt/parse/reblast/OneStep/Cau/tmp/supergroup_gap200_len900_cover0.5/SG_4/tmp/SG_4_R_84.fa'
# pfam_db_file = '/lustre/home/xuyuxing/Database/Interpro/interproscan-5.39-77.0/data/pfam/32.0/pfam_a.hmm'
# output_file = '/lustre/home/xuyuxing/Work/Cuscuta_HGT/HGT_Nt/parse/reblast/OneStep/Cau/tmp/supergroup_gap200_len900_cover0.5/SG_4/tmp/SG_4_R_84.pfam.aa'
#
# # output
# transeq_output = genome_seq_file + ".transeq"
# hmmscan_output = transeq_output + ".hmmscan"
#
# transeq_cmd_string = 'transeq -frame 6 -sequence %s -outseq %s' % (genome_seq_file, transeq_output)
# cmd_run(transeq_cmd_string)
#
# hmmscan_cmd_string = 'hmmscan --domtblout %s %s %s' % (hmmscan_output, pfam_db_file, transeq_output)
# cmd_run(hmmscan_cmd_string)
#
# output_dir = get_aa_seq(genome_seq_file, transeq_output, hmmscan_output)
#
# with open(output_file, 'w') as f:
#     for seq in output_dir:
#         for strand in output_dir[seq]:
#             if not output_dir[seq][strand] == "":
#                 if strand == "+":
#                     f.write(">%s_p\n%s\n" % (seq, output_dir[seq][strand]))
#                 elif strand == "-":
#                     f.write(">%s_n\n%s\n" % (seq, output_dir[seq][strand]))
